package org.exoplatform.portlets.content.actions.component;

import java.security.AccessControlException;
import java.util.Map;

import javax.faces.event.ActionListener;
import javax.jcr.AccessDeniedException;
import javax.jcr.Node;

import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.templates.TemplateService;

public class UIActionForm extends UISimpleMulltipartForm implements
    CMSDialogComponent {

  protected TemplateService templateService_;
  protected ActionServiceContainer actionsServiceContainer_;
  protected Node parentNode_;
  protected Node actionNode_;
  protected String nodeType_;

  public UIActionForm(TemplateService templateService,
      ActionServiceContainer actionsServiceContainer) {
    super("workflowForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    templateService_ = templateService;
    actionsServiceContainer_ = actionsServiceContainer;

    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }

  public void createNewAction(Node parentNode, String actionType) throws Exception {
    clean();
    parentNode_ = parentNode;
    actionNode_ = null;
    nodeType_ = actionType;
  }

  public Node getNode() throws Exception {
    return actionNode_;
  }

  public boolean isNodeTypeSupported() throws Exception {
    return templateService_.isNodeTypeConfigured(nodeType_);
  }

  public String getDialogPath() throws Exception{
    return templateService_.getTemplatePath(true, nodeType_);
  }

  public void clean() {
    clearInputs();
    getChildren().clear();
    ActionListener[] listeners = getActionListeners();
    for (int i = 0; i < listeners.length; i++) {
      ExoActionListener listener = (ExoActionListener) listeners[i];
      String actionName = listener.getActionName();
      if (!CANCEL_ACTION.equals(actionName) && !SAVE_ACTION.equals(actionName))
        removeActionListener(listener);
    }
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionForm uiForm = (UIActionForm) event.getSource();
      Information iprovider = findInformationProvider(uiForm);
      Map sortedInputs = CMSUtils.prepareMap(uiForm.getUIInputs());
      sortedInputs.put("/node", sortedInputs.get("/node/exo:name"));
      try {
        try {
          uiForm.parentNode_.getSession().checkPermission(
              uiForm.parentNode_.getPath(), "add_node,set_property");
        } catch (AccessControlException e) {
            throw new AccessDeniedException(e.getMessage());
        }
        uiForm.actionsServiceContainer_.addAction(uiForm.parentNode_,
            uiForm.nodeType_, sortedInputs);
        ((UIExoComponent) uiForm.getParent())
            .setRenderedSibling(UIActionList.class);
        uiForm.clean();
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionForm uiForm = (UIActionForm) event.getSource();
      ((UIExoComponent) uiForm.getParent())
          .setRenderedSibling(UIActionList.class);
      uiForm.clean();
    }
  }

}
