package org.exoplatform.portlets.content.actions.component;

import java.util.List;

import javax.jcr.Node;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;

public class UIActionList extends UIExoCommand implements ChangeNodeListener{
  
  final static public String ADD_ACTION = "add".intern();  
  
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  public static Parameter[] VIEW_PARAMS = { new Parameter(ACTION , VIEW_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;

  protected ActionServiceContainer actionServiceContainer_;

  protected Node node_;
  
  public UIActionList(ActionServiceContainer actionServiceContainer) {
    setId("UIactionList") ;
    setRendererType("VelocityRenderer") ;
    
    actionServiceContainer_ = actionServiceContainer;        
    
    addActionListener(AddActionListener.class, ADD_ACTION) ;    
    addActionListener(ViewActionListener.class, VIEW_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }

  public void  onChange(UIJCRExplorer uiExplorer, Node node)  throws Exception {
    setRendered(false) ;
    node_ = node;
  }      
  
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }  
  public Parameter[] getBackParams() { return CANCEL_PARAMS ; }
  
  public boolean hasActions() throws Exception{
     return actionServiceContainer_.hasActions(node_);
  }
  
  public List<Node> getActions() throws Exception{
    return actionServiceContainer_.getActions(node_);
  }

  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionList uicomp = (UIActionList) event.getComponent() ;
      uicomp.setRenderedSibling(UIActionFormController.class);
      UIActionFormController actionFormController = (UIActionFormController) uicomp.getSibling(
          UIActionFormController.class);
      actionFormController.showNewActionForm();
    }
  }    

  static public class ViewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionList uicomp = (UIActionList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      UIActionFormController actionFormController = (UIActionFormController) uicomp.getSibling(
          UIActionFormController.class);
      actionFormController.showActionForm(name);
      uicomp.setRenderedSibling(UIActionFormController.class);
    }
  }        
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionList uicomp = (UIActionList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.actionServiceContainer_.removeAction(uicomp.node_, name);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }        
}
