package org.exoplatform.portlets.content.display.model;

import java.util.Enumeration;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.exoplatform.portlets.content.display.dynamic.component.ContentConfigConstants;
import org.exoplatform.services.portletcontainer.ExoPortletRequest;

public class ContentConfigManipulationsUtils implements ContentConfigConstants {

  public static int getNewPosition() {
    ExternalContext eContext = FacesContext.getCurrentInstance()
        .getExternalContext();
    PortletRequest request = (PortletRequest) eContext.getRequest();
    PortletPreferences prefs = request.getPreferences();
    Enumeration names = prefs.getNames();
    int i = 0;
    while (names.hasMoreElements()) {
      names.nextElement();
      i++;
    }
    return i - 1;
  }

  public static void saveContentConfigButDoNotStore(PortletPreferences prefs,
      ContentConfig config) throws Exception {
    String uri = config.getUri();
    if (uri == null) {
      uri = "";
      config.setUri(uri);
    }

    String name = config.getName();
    String[] tmp = new String[4];
    tmp[0] = "uri=" + uri;
    tmp[1] = "configType=" + config.getConfigType();
    tmp[2] = "documentType=" + config.getDocumentType();
    tmp[3] = "position=" + config.getPosition();

    prefs.setValues(name, tmp);
  }

  public static void saveContentConfig(ContentConfig config) throws Exception {
    FacesContext context = FacesContext.getCurrentInstance();
    ExoPortletRequest request = (ExoPortletRequest) context
        .getExternalContext().getRequest();
    PortletPreferences prefs = request.getPreferences();
    saveContentConfigButDoNotStore(prefs, config);
    prefs.store();
  }

  public static void removeContentConfig(String name) throws Exception {
    FacesContext context = FacesContext.getCurrentInstance();
    ExoPortletRequest request = (ExoPortletRequest) context
        .getExternalContext().getRequest();
    PortletPreferences prefs = request.getPreferences();
    String[] values = prefs.getValues(name, DEFAULT_VALUES);
    ContentConfig contentConfig = new ContentConfig(name, values);
    int pos = new Integer(contentConfig.getPosition()).intValue();
    
    Enumeration e = prefs.getNames();
    while (e.hasMoreElements()) {
      String prefName = (String) e.nextElement();
      if (!LAYOUT.equals(prefName)) {
        values = prefs.getValues(prefName, DEFAULT_VALUES);
        ContentConfig currentContentConfig = new ContentConfig(prefName, values);
        int pos1 = new Integer(currentContentConfig.getPosition()).intValue();
        if (pos1 > pos) {
          int newPos = pos1 - 1;
          currentContentConfig.setPosition("" + newPos);
          saveContentConfigButDoNotStore(prefs, currentContentConfig);
        }
      }
    }
    prefs.reset(name);
    prefs.store();
  }
  
  public static void incrementConfigPosition(String configName) throws Exception{
    processInOrDecrement(configName, true);
  }
  
  public static void decrementConfigPosition(String configName) throws Exception {
    processInOrDecrement(configName, false);
  }  

  private static void processInOrDecrement(String configName, boolean increment) throws Exception{
    FacesContext context = FacesContext.getCurrentInstance();
    ExoPortletRequest request = (ExoPortletRequest) context
        .getExternalContext().getRequest();
    PortletPreferences prefs = request.getPreferences();
    String[] values = prefs.getValues(configName, DEFAULT_VALUES);
    ContentConfig contentConfig = new ContentConfig(configName, values);
    int pos = new Integer(contentConfig.getPosition()).intValue();
        
    Enumeration e = prefs.getNames();
    while (e.hasMoreElements()) {
      String prefName = (String) e.nextElement();
      if (!LAYOUT.equals(prefName)) {
        values = prefs.getValues(prefName, DEFAULT_VALUES);
        ContentConfig currentContentConfig = new ContentConfig(prefName, values);
        int pos1 = new Integer(currentContentConfig.getPosition()).intValue();
        
        int positionToTest = 0;
        if(increment)
          positionToTest = pos + 1;
        else
          positionToTest = pos - 1;
        
        if(pos1 == positionToTest){
          int newPos = pos;
          currentContentConfig.setPosition("" + newPos);
          saveContentConfigButDoNotStore(prefs, currentContentConfig);
          contentConfig.setPosition("" + pos1);
          saveContentConfigButDoNotStore(prefs, contentConfig);
          break;
        }
      }
    }
    prefs.store();    
  }
  
  
}
