/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.security.AccessControlException;
import java.util.Map;
import java.util.ResourceBundle;

import javax.jcr.AccessDeniedException;
import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;

/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIContentDisplayer.java,v 1.2 2004/08/07 18:11:26 tuan08 Exp $
 */
public class UIDocumentForm extends UISimpleMulltipartForm implements
    CMSDialogComponent {

  String documentType_;
  String documentName_;
  protected TemplateService templateService_;
  protected CmsService cmsService_;
  protected UIStringInput nameInput_;
  protected Node node_;

  public UIDocumentForm(CmsService cmsService, TemplateService templateService)
      throws Exception {
    super("documentForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    nameInput_ = new UIStringInput("docName", "");
    cmsService_ = cmsService;
    templateService_ = templateService;
    addActionListener(StoreDocumentActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  public void editDocument(String name) throws Exception {
    clean();
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    node_ = uiExplorer.getSelectNode().getNode(name);
    UIInput input = getUIInput("docName");
    input.setEditable(false);
    input.setValue(node_.getName());
    documentType_ = node_.getProperty("jcr:primaryType").getString();
  }

  public void showNewDocument(String documentType) throws Exception {
    clean();
    UIInput input = getUIInput("docName");
    input.setEditable(true);
    node_ = null;
    documentType_ = documentType;
  }
  
  public boolean isCheckedOut() throws RepositoryException {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Node node = uiExplorer.getSelectNode();
    return node.isCheckedOut();
  }

  public String getDocumentType() {
    return documentType_;
  }

  public String getDialogPath() throws Exception {
    return templateService_.getTemplatePath(true, documentType_);
  }

  public Node getNode() throws Exception {
    return node_;
  }

  public boolean isDocumentTypeSupported() throws Exception {
    return templateService_.isNodeTypeConfigured(documentType_);
  }

  public void clean() {
    clearInputs();
    getChildren().clear();
    nameInput_.setValue("");
    add(nameInput_);
  }

  static public class StoreDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;      
      UIDocumentForm uiForm = (UIDocumentForm) event.getSource();
      String name = uiForm.nameInput_.getValue();
      Information iprovider = findInformationProvider(uiForm);
      if (name == null || name.length() == 0) {
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIDocumentForm.msg.name-is-null")));
        return;
      }
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm
          .getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode();
      Map sortedInputs = CMSUtils.prepareMap(uiForm.getUIInputs());

      String rootPath = "/node";
      JcrInputProperty property = new JcrInputProperty();
      property.setJcrPath(rootPath);
      property.setValue(name);
      sortedInputs.put(rootPath, property);
      try {
        try {
          node.getSession().checkPermission(node.getPath(),
            "add_node,set_property");
        } catch (AccessControlException e) {
          throw new AccessDeniedException(e.getMessage());
        }
        uiForm.cmsService_.storeNode(uiForm.documentType_, node, sortedInputs);
        uiExplorer.broadcastOnChange(node);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentForm uiForm = (UIDocumentForm) event.getComponent() ;
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm
          .getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode();
      uiExplorer.broadcastOnChange(node);      
    }
  }
 
}