package org.exoplatform.portlets.content.script.component;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.scripts.ScriptService;

public class UIScriptForm extends UISimpleMulltipartForm {

  static String SCRIPT_NAME = "script".intern(), CONTENT = "content".intern() ;
  
  protected ScriptService scriptService_;
  protected String scriptName_;
  protected UIStringInput scriptNameInput_;
  protected UITextArea scriptContentInput_;

  public UIScriptForm(ScriptService scriptService) {
    super("scriptForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    
    scriptService_ = scriptService;
    
    scriptNameInput_ = new UIStringInput(SCRIPT_NAME, ""); 
    scriptContentInput_ = new UITextArea(CONTENT, "");
    scriptContentInput_.setClazz("large");
    
    add(scriptNameInput_) ;
    add(scriptContentInput_) ;    
    
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);    
  }
  
  public void setScriptName(String scriptName){
    scriptName_ = scriptName;
    try {
      String script =  scriptService_.getScriptAsText(scriptName_);
      scriptNameInput_.setText(scriptName_).setEditable(false);
      scriptContentInput_.setText(script);
    } catch (Exception e){
      scriptNameInput_.setText("").setEditable(true);
      scriptContentInput_.setText("");
    }     
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptForm uiForm = (UIScriptForm) event.getSource();
      String name = uiForm.scriptNameInput_.getText();
      String text = uiForm.scriptContentInput_.getText();
      Information iprovider = findInformationProvider(uiForm);      
      try {
      uiForm.scriptService_.addScript(name, text);
      uiForm.setRenderedSibling(UIScriptList.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }    
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptForm uiForm = (UIScriptForm) event.getSource();
      uiForm.setRenderedSibling(UIScriptList.class);      
    }
  }  
  
}
