/*
 * Created on Mar 22, 2005
 */
package org.exoplatform.services.cms.publications.impl;

import java.io.InputStream;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.commons.logging.Log;
import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.impl.Utils;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.resources.LocaleConfig;
import org.exoplatform.services.resources.LocaleConfigService;
import org.picocontainer.Startable;

/**
 * @author benjaminmestrallet
 */
public class PublicationsServiceImpl implements Startable {

  private ConfigurationManager cservice_;
  private RepositoryService jcrService_;
  private CmsConfigurationService cmsConfigurationService_;
  private Log log_;
  private PredefinedPublicationsConfig config_;
  private Collection localeConfigs_;

  public PublicationsServiceImpl(ConfigurationManager cservice,
      LocaleConfigService localeService, RepositoryService jcrService,
      CmsConfigurationService cmsConfigurationService, LogService logService,
      ActionServiceContainer actionServiceContainer, InitParams params) throws Exception {
    cservice_ = cservice;
    jcrService_ = jcrService;
    cmsConfigurationService_ = cmsConfigurationService;
    log_ = logService.getLog(getClass().getName());
    config_ = (PredefinedPublicationsConfig) params.getObjectParamValues(
        PredefinedPublicationsConfig.class).get(0);
    localeConfigs_ = localeService.getLocalConfigs();
  }

  public void start() {
    try {
      fillWorkspaces(cmsConfigurationService_.getDraftWorkspace(), false);
      fillWorkspaces(cmsConfigurationService_.getBackupWorkspace(), false);
      fillWorkspaces(cmsConfigurationService_.getWorkspace(), true);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void stop() {
  }

  private void fillWorkspaces(String workspace, boolean importFiles)
      throws Exception {
    ManageableRepository repository = jcrService_.getRepository();

    String publicationPath = cmsConfigurationService_
        .getJcrPath(BasePath.CMS_PUBLICATIONS_PATH);

    Session session = null;
    try {
      session = repository.getSystemSession(workspace);
    } catch (RepositoryException re) {
      // no production and draft workspace configure == no cms for the portal
      // instance
      return;
    }

    LocaleConfig firstLocaleConfig = (LocaleConfig) localeConfigs_.iterator()
        .next();
    try {
      session.getItem(publicationPath + "/" + firstLocaleConfig.getLanguage());
      return;
    } catch (PathNotFoundException e) {
    }

    List publications = config_.getPublications();
    Node publicationHomeNode = (Node) session.getItem(publicationPath);
    for (int i = 0; i < publications.size(); i++) {
      PredefinedPublicationsConfig.Publication publication = (PredefinedPublicationsConfig.Publication) publications
          .get(i);
      for (Iterator iterator = localeConfigs_.iterator(); iterator.hasNext();) {
        LocaleConfig localeConfig = (LocaleConfig) iterator.next();
        String locale = localeConfig.getLanguage();
        Node localisedNode = Utils.makePath(publicationHomeNode, locale,
            "nt:unstructured");
        Node publicationNode = Utils.makePath(localisedNode, publication
            .getPath(), "nt:unstructured");
        if (importFiles)
          importInJCR(publication.getReferencedFiles(), publicationNode, locale);
      }
    }
    publicationHomeNode.save();
  }

  private void importInJCR(List files, Node publicationNode, String locale)
      throws PathNotFoundException, RepositoryException {
    for (Iterator iterator = files.iterator(); iterator.hasNext();) {
      String file = (String) iterator.next();
      String warPath = cmsConfigurationService_.getContentLocation()
          + publicationNode.getPath() + "/" + file;
      try {
        InputStream iS = cservice_.getInputStream(warPath);
        Node fileNode = publicationNode.addNode(file, "nt:file");
        Node contentNode = fileNode.addNode("jcr:content", "nt:resource");
        contentNode.setProperty("jcr:encoding", "UTF-8");
        contentNode.setProperty("jcr:data", iS);
        contentNode.setProperty("jcr:mimeType", "text/html");
        contentNode.setProperty("jcr:lastModified", new GregorianCalendar(
            new Locale(locale)));
      } catch (Exception e1) {
        // means the version of the file for that locale does not exist
      }
    }
  }

}
