/*
 * Created on Mar 1, 2005
 */
package org.exoplatform.services.cms.templates.impl;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.impl.Utils;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.security.SecurityService;
import org.picocontainer.Startable;

/**
 * @author benjaminmestrallet
 */
public class TemplateServiceImpl implements TemplateService, Startable {

  private RepositoryService jcrService_;

  private ConfigurationManager cservice_;

  private CmsConfigurationService cmsConfigurationService_;

  private SecurityService securityService_;

  private List<TemplatePlugin> plugins_;

  public TemplateServiceImpl(RepositoryService jcrService,
      ConfigurationManager cservice, InitParams params,
      CmsConfigurationService cmsConfigurationService,
      SecurityService securityService) throws Exception {
    cmsConfigurationService_ = cmsConfigurationService;
    securityService_ = securityService;
    jcrService_ = jcrService;
    cservice_ = cservice;
    plugins_ = new ArrayList<TemplatePlugin>();
  }

  public void start() {
    try {
      initRepository();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void stop() {
  }

  public void addTemplates(ComponentPlugin plugin) {
    if (plugin instanceof TemplatePlugin)
      plugins_.add((TemplatePlugin) plugin);
  }

  private void initRepository() throws Exception {
    // use a anonymous connection for the configuration as the user is not
    // authentified at that time
    ManageableRepository jcrRepository = jcrService_.getRepository();
    Session session = null;
    try {
      session = jcrRepository.getSystemSession(cmsConfigurationService_
          .getWorkspace());
    } catch (RepositoryException re) {
      return;
    }

    Node root = session.getRootNode();

    List nodetypes = null;
    for (int j = 0; j < plugins_.size(); j++) {
      nodetypes = plugins_.get(j).getNodeTypes();

      String templatesPath = cmsConfigurationService_
          .getJcrPath(BasePath.CMS_TEMPLATES_PATH);
      if (nodetypes.isEmpty())
        return;
      TemplateConfig.NodeType nodeType = (TemplateConfig.NodeType) nodetypes
          .iterator().next();
      String nodeTypeName = nodeType.getNodetypeName();
      try {
        session.getItem(templatesPath + "/" + nodeTypeName);
        return;
      } catch (PathNotFoundException e) {
      }

      Node templatesHome = Utils.makePath(root, templatesPath,
          "nt:unstructured");
      String warPath = cmsConfigurationService_.getContentLocation()
          + templatesPath;

      for (Iterator iter = nodetypes.iterator(); iter.hasNext();) {
        nodeType = (TemplateConfig.NodeType) iter.next();
        nodeTypeName = nodeType.getNodetypeName();
        Node nodeTypeHome = null;
        if (!templatesHome.hasNode(nodeTypeName))
          nodeTypeHome = Utils.makePath(templatesHome, nodeTypeName,
              "nt:unstructured");
        else
          nodeTypeHome = templatesHome.getNode(nodeTypeName);

        List dialogs = nodeType.getReferencedDialog();
        Node dialogsHome = Utils.makePath(nodeTypeHome, DIALOGS,
            "nt:unstructured");
        addNode(warPath, dialogsHome, dialogs);

        List views = nodeType.getReferencedView();
        Node viewsHome = Utils.makePath(nodeTypeHome, VIEWS, "nt:unstructured");
        addNode(warPath, viewsHome, views);
      }
    }
    root.save();
  }

  private void addNode(String basePath, Node nodeTypeHome, List templates)
      throws Exception {
    for (Iterator iterator = templates.iterator(); iterator.hasNext();) {
      TemplateConfig.Template template = (TemplateConfig.Template) iterator
          .next();
      String templateFileName = template.getTemplateFile();
      String path = basePath + "/" + templateFileName;
      InputStream in = cservice_.getInputStream(path);
      String nodeName = templateFileName.substring(templateFileName
          .lastIndexOf("/") + 1, templateFileName.indexOf("."));

      Node contentNode = nodeTypeHome.addNode(nodeName, "exo:template");
      contentNode.setProperty("exo:roles", template.getParsedRoles());
      contentNode.setProperty("exo:templateFile", in);
    }
  }

  private Node getTemplatesHome() throws Exception {
    ManageableRepository jcrRepository = jcrService_.getRepository();
    Session session = null;
    try {
      session = jcrRepository.getSystemSession(cmsConfigurationService_
          .getWorkspace());
    } catch (RepositoryException re) {
      return null;
    }

    String templatesPath = cmsConfigurationService_
        .getJcrPath(BasePath.CMS_TEMPLATES_PATH);
    try {
      return (Node) session.getItem(templatesPath);
    } catch (PathNotFoundException e) {
    }
    return null;
  }

  private Node getNodeTypeNode(String nodeTypeName) throws Exception {
    ManageableRepository jcrRepository = jcrService_.getRepository();
    Session session = null;
    try {
      session = jcrRepository.getSystemSession(cmsConfigurationService_
          .getWorkspace());
    } catch (RepositoryException re) {
      return null;
    }

    String templatesPath = cmsConfigurationService_
        .getJcrPath(BasePath.CMS_TEMPLATES_PATH);
    try {
      return (Node) session.getItem(templatesPath + "/" + nodeTypeName);
    } catch (PathNotFoundException e) {
    }
    return null;
  }

  public boolean isNodeTypeConfigured(String nodeTypeName) throws Exception {
    NodeIterator iter = getDocumentTypes();
    while (iter.hasNext()) {
      Node node = iter.nextNode();
      if (node.getName().equals(nodeTypeName))
        return true;
    }
    return false;
  }

  public NodeIterator getTemplates(boolean isDialog, String nodeTypeName)
      throws Exception {
    Node nodeTypeHome = getNodeTypeNode(nodeTypeName);
    if (isDialog)
      return nodeTypeHome.getNode(DIALOGS).getNodes();
    else
      return nodeTypeHome.getNode(VIEWS).getNodes();
  }

  public NodeIterator getDocumentTypes() throws Exception {
    Session session;
    try {
      session = getSystemSession(cmsConfigurationService_.getWorkspace());
    } catch (Exception e) {
      return null;
    }
    String templatesPath = cmsConfigurationService_
        .getJcrPath(BasePath.CMS_TEMPLATES_PATH);
    Node templatesHome = (Node) session.getItem(templatesPath);
    return templatesHome.getNodes();
  }

  private Session getSystemSession(String workspace)
      throws RepositoryException, RepositoryConfigurationException {
    ManageableRepository jcrRepository = jcrService_.getRepository();
    return jcrRepository.getSystemSession(workspace);
  }

  public String getDefaultTemplatePath(boolean isDialog, String nodeTypeName) {
    if (isDialog)
      return cmsConfigurationService_.getJcrPath(BasePath.CMS_TEMPLATES_PATH)
          + "/" + nodeTypeName + DEFAULT_DIALOGS_PATH;
    else
      return cmsConfigurationService_.getJcrPath(BasePath.CMS_TEMPLATES_PATH)
          + "/" + nodeTypeName + DEFAULT_VIEWS_PATH;
  }

  private Node getTemplateNode(boolean isDialog, String nodeTypeName,
      String templateName) throws Exception {
    String type = DIALOGS;
    if (!isDialog)
      type = VIEWS;
    Node nodeTypeNode = getNodeTypeNode(nodeTypeName);
    return nodeTypeNode.getNode(type).getNode(templateName);
  }

  private Node getTemplateNode(boolean isDialog, String nodeTypeName)
      throws Exception {
    String type = DIALOGS;
    if (!isDialog)
      type = VIEWS;
    String userId = SessionContainer.getInstance().getRemoteUser();
    Node nodeTypeNode = getNodeTypeNode(nodeTypeName);
    NodeIterator templatessIter = nodeTypeNode.getNode(type).getNodes();
    Node selectedTemplateNode = null;
    while (templatessIter.hasNext()) {
      Node node = templatessIter.nextNode();
      Value[] roles = node.getProperty("exo:roles").getValues();
      for (int i = 0; i < roles.length; i++) {
        String templateRole = roles[i].getString();
        if ("*".equals(templateRole)) {
          selectedTemplateNode = node;
        } else if (securityService_.hasMembershipInGroup(userId, templateRole))
          return node;
      }
    }
    return selectedTemplateNode;
  }

  public String getTemplatePath(boolean isDialog, String nodeTypeName)
      throws Exception {
    Node templateNode = getTemplateNode(isDialog, nodeTypeName);
    return templateNode.getPath();
  }

  public String getTemplatePath(boolean isDialog, String nodeTypeName,
      String templateName) throws Exception {
    Node templateNode = getTemplateNode(isDialog, nodeTypeName, templateName);
    return templateNode.getPath();
  }

  public String getTemplate(boolean isDialog, String nodeTypeName)
      throws Exception {
    Node templateNode = getTemplateNode(isDialog, nodeTypeName);
    return templateNode.getProperty("exo:templateFile").getString();
  }

  public String getTemplate(boolean isDialog, String nodeTypeName,
      String templateName) throws Exception {
    Node templateNode = getTemplateNode(isDialog, nodeTypeName, templateName);
    return templateNode.getProperty("exo:templateFile").getString();
  }

  public void removeTemplate(boolean isDialog, String nodeTypeName,
      String templateName) throws Exception {
    Node templatesHome = getTemplatesHome();
    Node nodeTypeHome = templatesHome.getNode(nodeTypeName);
    Node specifiedTemplatesHome = null;
    if (isDialog) {
      specifiedTemplatesHome = nodeTypeHome.getNode(DIALOGS);
    } else {
      specifiedTemplatesHome = nodeTypeHome.getNode(VIEWS);
    }
    Node contentNode = specifiedTemplatesHome.getNode(templateName);
    contentNode.remove();
    nodeTypeHome.save();
  }

  public void addTemplate(boolean isDialog, String nodeTypeName,
      String templateName, String[] roles, String templateFile)
      throws Exception {

    Node templatesHome = getTemplatesHome();

    Node nodeTypeHome = null;
    if (!templatesHome.hasNode(nodeTypeName))
      nodeTypeHome = Utils.makePath(templatesHome, nodeTypeName,
          "nt:unstructured");
    else
      nodeTypeHome = templatesHome.getNode(nodeTypeName);

    Node specifiedTemplatesHome = null;
    if (isDialog) {
      if (!nodeTypeHome.hasNode(DIALOGS)) {
        specifiedTemplatesHome = Utils.makePath(nodeTypeHome, DIALOGS,
            "nt:unstructured");
      } else {
        specifiedTemplatesHome = nodeTypeHome.getNode(DIALOGS);
      }
    } else {
      if (!nodeTypeHome.hasNode(VIEWS)) {
        specifiedTemplatesHome = Utils.makePath(nodeTypeHome, VIEWS,
            "nt:unstructured");
      } else {
        specifiedTemplatesHome = nodeTypeHome.getNode(VIEWS);
      }
    }

    Node contentNode = null;
    if (specifiedTemplatesHome.hasNode(templateName)) {
      contentNode = specifiedTemplatesHome.getNode(templateName);
    } else {
      contentNode = specifiedTemplatesHome
          .addNode(templateName, "exo:template");
    }
    contentNode.setProperty("exo:roles", roles);
    contentNode.setProperty("exo:templateFile", templateFile);

    templatesHome.save();
  }

}
