/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.impl.core.nodetype.nt;


import javax.jcr.nodetype.*;
import javax.jcr.PropertyType;
import javax.jcr.version.OnParentVersionAction;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import javax.jcr.RepositoryException;


/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 31 juil. 2004
 */
public class NodeTypeType extends NodeTypeImpl {

  public NodeTypeType(NodeTypeManager manager) throws NoSuchNodeTypeException,RepositoryException {

    super(manager);
    this.name = "nt:nodeType";
    this.mixin = false;
    this.orderableChild = false;
    this.primaryItemName = null;
    this.declaredSupertypes = new NodeType[1];
    this.declaredSupertypes[0] = manager.getNodeType("nt:base");
    this.declaredPropertyDefinitions = new PropertyDefinitionImpl[5];
    this.declaredChildNodeDefinitions = new NodeDefinition[2];

    this.declaredPropertyDefinitions[0] = new PropertyDefinitionImpl(
        "jcr:nodeTypeName",           //name
        this,                         //declaringNodeType
        PropertyType.NAME  ,          //requiredType
        EMPTY_CONSTRAINTS,             //valueConstraints 
        null,                        //defaultValues
        false,                         //autoCreate
        true,                         //mandatory        
        OnParentVersionAction.COPY,   //onVersion        
        false,                        //readOnly         
        false                         //multiple         
        );

    this.declaredPropertyDefinitions[1] = new PropertyDefinitionImpl(
        "jcr:supertypes",             //name
        this,                         //declaringNodeType
        PropertyType.NAME,            //requiredType
        EMPTY_CONSTRAINTS,             //valueConstraints 
        null,                         //defaultValues
        false,                        //autoCreate
        false,                        //mandatory
        OnParentVersionAction.COPY,   //onVersion        
        false,                        //readOnly         
        true                          //multiple         
        );

    this.declaredPropertyDefinitions[2] = new PropertyDefinitionImpl(
        "jcr:isMixin",                //name
        this,                         //declaringNodeType
        PropertyType.BOOLEAN,         //requiredType
        EMPTY_CONSTRAINTS,             //valueConstraints 
        null,                         //defaultValues
        false,                         //autoCreate
        true ,                        //mandatory
        OnParentVersionAction.COPY,   //onVersion        
        false,                        //readOnly         
        false                          //multiple
        );

    this.declaredPropertyDefinitions[3] = new PropertyDefinitionImpl(
        "jcr:hasOrderableChildNodes", //name
        this,                         //declaringNodeType
        PropertyType.BOOLEAN,         //requiredType
        EMPTY_CONSTRAINTS,            //valueConstraints 
        null,                         //defaultValues
        false,                         //autoCreate
        true ,                        //mandatory
        OnParentVersionAction.COPY,   //onVersion        
        false,                        //readOnly         
        false                          //multiple
        );

    this.declaredPropertyDefinitions[4] = new PropertyDefinitionImpl(
        "jcr:primaryItemName",         //name
        this,                         //declaringNodeType
        PropertyType.NAME,            //requiredType
        EMPTY_CONSTRAINTS,            //valueConstraints 
        null,                  //defaultValues
        false,                         //autoCreate
        false,                        //mandatory
        OnParentVersionAction.COPY,   //onVersion        
        false,                        //readOnly         
        false                          //multiple
        );


    NodeType[] types = {manager.getNodeType("nt:propertyDefinition")};
    this.declaredChildNodeDefinitions[0] = new NodeDefinitionImpl(
        "jcr:propertyDefinition",         // name
        this,                             // declaringNodeType
        types,                            // requiredNodeTypes
        manager.getNodeType("nt:propertyDefinition"),  // defaultNodeType
        false,                            // autoCreate       
        false,                            // mandatory        
        OnParentVersionAction.VERSION,    // onVersion        
        false,                            // protected        
        true                              // same-name sibling
        );

    NodeType[] reqNodeTypes = {manager.getNodeType("nt:childNodeDefinition")};

    this.declaredChildNodeDefinitions[1] = new NodeDefinitionImpl(
        "jcr:childNodeDefinition",               // name
        this,                             // declaringNodeType
        reqNodeTypes,                            // requiredNodeTypes
        manager.getNodeType("nt:childNodeDefinition"),  // defaultNodeType
        false,                            // autoCreate       
        false,                            // mandatory        
        OnParentVersionAction.VERSION,    // onVersion        
        false,                            // protected        
        true                              // same-name sibling
        );

  }

}
