/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.impl.core.nodetype.nt;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;

/**
 * Created y the eXo platform team User: Benjamin Mestrallet Date: 31 juil. 2004
 */
public class PropertyDefType extends NodeTypeImpl {

  public PropertyDefType(NodeTypeManager manager)
      throws NoSuchNodeTypeException, RepositoryException {
    super(manager);
    this.name = "nt:propertyDefinition";
    this.mixin = false;
    this.orderableChild = false;
    this.primaryItemName = null;
    this.declaredSupertypes = new NodeType[1];
    this.declaredSupertypes[0] = manager.getNodeType("nt:base");
    this.declaredPropertyDefinitions = new PropertyDefinition[9];

    this.declaredPropertyDefinitions[0] = new PropertyDefinitionImpl(
        "jcr:name", // name
        this, // declaringNodeType
        PropertyType.NAME, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        false, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        false // multiple
    );

    this.declaredPropertyDefinitions[1] = new PropertyDefinitionImpl(
        "jcr:requiredType", // name
        this, // declaringNodeType
        PropertyType.STRING, // requiredType
        // EMPTY_CONSTRAINTS, //valueConstraints
        new String[] { "STRING", "BINARY", "LONG", "DOUBLE", "BOOLEAN", "DATE",
            "NAME", "PATH", "REFERENCE", "UNDEFINED" }, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        false // multiple
    );

    this.declaredPropertyDefinitions[2] = new PropertyDefinitionImpl(
        "jcr:valueConstraints", // name
        this, // declaringNodeType
        PropertyType.STRING, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        false, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        true // multiple
    );

    this.declaredPropertyDefinitions[3] = new PropertyDefinitionImpl(
        "jcr:defaultValues", // name
        this, // declaringNodeType
        PropertyType.UNDEFINED, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        false, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        true // multiple
    );

    this.declaredPropertyDefinitions[4] = new PropertyDefinitionImpl(
        "jcr:autoCreated", // name
        this, // declaringNodeType
        PropertyType.BOOLEAN, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        false // multiple
    );

    this.declaredPropertyDefinitions[5] = new PropertyDefinitionImpl(
        "jcr:mandatory", // name
        this, // declaringNodeType
        PropertyType.BOOLEAN, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        false // multiple
    );

    this.declaredPropertyDefinitions[6] = new PropertyDefinitionImpl(
        "jcr:onParentVersion", // name
        this, // declaringNodeType
        PropertyType.STRING, // requiredType
        new String[] { "COPY", "VERSION", "INITIALIZE", "COMPUTE", "IGNORE",
            "ABORT" }, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        false // multiple
    );

    this.declaredPropertyDefinitions[7] = new PropertyDefinitionImpl(
        "jcr:protected", // name
        this, // declaringNodeType
        PropertyType.BOOLEAN, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        false // multiple
    );

    this.declaredPropertyDefinitions[8] = new PropertyDefinitionImpl(
        "jcr:multiple", // name
        this, // declaringNodeType
        PropertyType.BOOLEAN, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.COPY, // onVersion
        false, // readOnly
        false // multiple
    );

  }

}
