/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.value;

import javax.jcr.ValueFormatException;

import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.storage.TransientValueData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: PermissionValue.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class PermissionValue extends BaseValue {

  private static final int TYPE = ExtendedPropertyType.PERMISSION;

  private String identity;

  private String permission;

  public PermissionValue(ValueData data) {
    super(TYPE, data);
  }

  public PermissionValue(String identity, String permission) {
    super(TYPE, new TransientValueData(identity + " " + permission));
    if (identity.indexOf(" ") != -1)
      throw new RuntimeException("Identity should not contain ' '");
    if(!permission.equals(PermissionType.READ) &&
       !permission.equals(PermissionType.ADD_NODE) &&
       !permission.equals(PermissionType.REMOVE) &&
       !permission.equals(PermissionType.SET_PROPERTY))
      throw new RuntimeException("Permission should be one of defined in PermissionType. Have "+permission);
    this.identity = identity;
    this.permission = permission;
  }

  protected String getInternalString() throws ValueFormatException {
    return identity + ":" + permission;
  }

  /**
   * @return Returns the identity.
   */
  public String getIdentity() {
    return identity;
  }

  /**
   * @return Returns the permission
   */
  public String getPermission() {
    return permission;
  }
}