/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.util.Stack;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.util.TraversingItemVisitor;
import javax.jcr.version.OnParentVersionAction;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: FrozenNodeInitializer.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class FrozenNodeInitializer extends TraversingItemVisitor {

  private Stack contextNodes;
  
  //private Node rootNode; 

  //	private SessionDataManager manager;

  public FrozenNodeInitializer(NodeImpl frozen) throws RepositoryException {
    
    contextNodes = new Stack();
    contextNodes.push(frozen);
    //this.rootNode = rootNode;
    //System.out.println(">>>>>>>> frozen " + frozen.getPath()+" "+currentNode().getPath());
  }

  protected void entering(Property property, int level)
  throws RepositoryException {
    
    PropertyImpl frozenProperty = null;
    if (property.getName().equals("jcr:primaryType") && level == 1)
      frozenProperty = currentNode().createChildProperty("jcr:frozenPrimaryType",
          new Value[] { property.getValue() }, PropertyType.NAME);
    else if (property.getName().equals("jcr:uuid") && level == 1)
      frozenProperty = currentNode().createChildProperty("jcr:frozenUuid",
          new Value[] { property.getValue() }, PropertyType.STRING);
    else if (property.getName().equals("jcr:mixinTypes") && level == 1)
      frozenProperty = currentNode().createChildProperty("jcr:frozenMixinTypes",
          property.getValues(), PropertyType.NAME);
    else if(currentNode() == null) {
        //property.getParent().getDefinition().getOnParentVersion() 
        //!= OnParentVersionAction.COPY) {
      // special behaviour of parent - ignore this;
      return;
    } else {
      PropertyImpl propertyImpl = (PropertyImpl) property;
      int action = property.getDefinition().getOnParentVersion();
      if (action == OnParentVersionAction.IGNORE)
        return;
      else if (action == OnParentVersionAction.ABORT)
        throw new VersionException("Property is aborted " + property.getPath());
      else if (action == OnParentVersionAction.COPY
          || action == OnParentVersionAction.VERSION
          || action == OnParentVersionAction.INITIALIZE
          || action == OnParentVersionAction.COMPUTE) {
        frozenProperty = currentNode().createChildProperty(property.getName(),
            propertyImpl.getValueArray(), property.getType());
        // COMPUTE - primary and mixin types
        // IITIALIZE - uuid and created
        // should be ok with just copy
      } else 
        throw new RepositoryException("Unknown OnParentVersion value "+action);
    }

    //System.out.println(">>>>>>>> entering prop " + property.getPath()+
    //    " cur: "+frozenProperty.getPath());

  }

  protected void entering(Node node, int level) throws RepositoryException {
    // this node is not taken in account
    if(level == 0)
      return;

    // ignored parent
    if(currentNode() == null) {
      contextNodes.push(null);
      return;
    }
      
    
    int action = node.getDefinition().getOnParentVersion();
    
    NodeImpl frozenNode = null; //(NodeImpl) node;
    if (action == OnParentVersionAction.IGNORE) {
      contextNodes.push(null);
//      return;
    } else if (action == OnParentVersionAction.ABORT) {
      throw new VersionException("Node is aborted " + node.getPath()); 
    } else if (action == OnParentVersionAction.COPY) {
        frozenNode = currentNode().createChildNode(node.getName(), 
          node.getPrimaryNodeType().getName(), true, true);
        contextNodes.push(frozenNode);
        System.out.println(">>>>>>>> entering node (COPY)" + node.getPath()+" "+action+" cur: "+frozenNode.getPath());
    } else if (action == OnParentVersionAction.VERSION) {
      if(node.isNodeType("mix:versionable")) {
        frozenNode =  currentNode().createChildNode(node.getName(), "nt:versionedChild",
          true, true);
        Value vh = node.getSession().getValueFactory().createValue(node.getVersionHistory());
        frozenNode.createChildProperty("jcr:childVersionHistory", new Value[] {vh},
          PropertyType.REFERENCE);
        contextNodes.push(null);
        System.out.println(">>>>>>>> entering reference (VERSION)" + node.getPath()+" "+action+" cur: "+frozenNode.getPath());
      } else { //behaviour of IGNORE
        contextNodes.push(null);
//        return;
      }
      
    } else if (action == OnParentVersionAction.INITIALIZE) {
      contextNodes.push(null);
//      return;
    } else if (action == OnParentVersionAction.COMPUTE) {
      contextNodes.push(null);
//      return;
    } else
      throw new RepositoryException("Unknown onParentVersion type "+action);


  }

  protected void leaving(Property property, int level)
      throws RepositoryException {
  }

  protected void leaving(Node node, int level) throws RepositoryException {
    contextNodes.pop();
  }

  private NodeImpl currentNode() {
    return (NodeImpl) contextNodes.peek();
  }
  

}