/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.util.Stack;

import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeType;
import javax.jcr.util.TraversingItemVisitor;

import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.itemvisitors.NodeCopyVisitor;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: RestoreVisitor.java 3547 2006-01-03 08:42:56Z geaz $
 */

class RestoreVisitor extends TraversingItemVisitor  {
  
  protected boolean removeExisting;
  protected Stack parents;
  protected NodeImpl context;
  //protected VersionStorageManager versionStorage;
  
//  RestoreVisitor(NodeImpl context, boolean removeExisting, 
//      VersionStorageManager versionStorage) throws RepositoryException {
  RestoreVisitor(NodeImpl context, boolean removeExisting) throws RepositoryException {
  
    super();
    this.parents = new Stack();
    this.context = context;
    parents.push(context);
    this.removeExisting = removeExisting;
    //this.versionStorage = versionStorage; 
  }
  
  protected void entering(Property property, int level)
  throws RepositoryException {

    //System.out.println("Visit property " + property.getPath()+" "+curParent());
    if (curParent() != null) {
      PropertyInfo info = new PropertyInfo(property);
      if (info.getName() != null) { // ignore if so
        Property p = curParent().createChildProperty(info.getName(),
            info.getValues(), info.getType());
        System.out.println("Restore property " + p.getPath() + " "
            + curParent().getPath() + " "
            + curParent().getPrimaryNodeType().getName());
      }
    }
      
  }
  
  protected void entering(Node node, int level) throws RepositoryException {
    if(node.isNodeType("nt:frozenNode") || curParent() == null) {
      // can be either root or referenced from versionedChild
      System.out.println("frozenNode "+node.getPath());
    } else if(node.isNodeType("nt:versionedChild")) {
      String ref = node.getProperty("jcr:childVersionHistory").getString();
      //VersionHistoryImpl history = versionStorage.getVersionHistoryByUUID(ref, (SessionImpl) node.getSession());
      VersionHistoryImpl history = (VersionHistoryImpl)((SessionImpl)node.getSession()).getNodeByUUID(ref);
      String versionableUuid = history.getVersionableUUID();
      
      System.out.println("versionedChild node>> "+node.getPath()+" "+versionableUuid);
      
      try {
        NodeImpl childNode = (NodeImpl)history.getSession().getNodeByUUID(versionableUuid);
        System.out.println("versionable child found >> "+childNode.getPath());
        if(removeExisting) {
          String name = childNode.getName();
          String ptName = childNode.getPrimaryNodeType().getName();
          String[] mtNames = childNode.getMixinTypeNames();

          NodeImpl newNode = curParent().createChildNode(name, ptName, mtNames, false);

          System.out.println("copy node >> parent: "+curParent().getPath());

          NodeCopyVisitor copyVisitor = new NodeCopyVisitor(newNode, false);
          childNode.accept(copyVisitor);
          
          parents.push(null);
          
        } else {
          // 2. exists and not a child and removeEx = false (ItemExistsException)
          throw new ItemExistsException("Node "+versionableUuid+" already exists "+childNode.getPath()+" and removeExisting==false"); 
        }
        
      } catch (ItemNotFoundException e) {
        System.out.println("versionable UUID not found >> "+versionableUuid+" "+node.getPath());
        // 3. not exists
        // gets last version (by time of creation)
        NodeImpl frozen = (NodeImpl)history.getLastVersion().getNode("jcr:frozenNode");
        String name = node.getName();
        String ptName = frozen.getProperty("jcr:frozenPrimaryType").getString();
        Value[] mtValues = frozen.getProperty("jcr:frozenMixinTypes").getValues();
        String[] mtNames = new String[mtValues.length];
        for(int i=0; i<mtNames.length; i++)
          mtNames[i] = mtValues[i].getString();
        String uuid = node.getProperty("jcr:frozenUuid").getString();
        NodeImpl restoredNode = curParent().createChildNode(name, ptName, mtNames, uuid, true, false);

//        RestoreVisitor childVisitor = new RestoreVisitor(restoredNode, 
//            removeExisting, versionStorage);
        RestoreVisitor childVisitor = new RestoreVisitor(restoredNode, removeExisting); 

        frozen.accept(childVisitor);
      } 
      
      
    } else { // ordinary node for copy under nt:frozenNode
      String name = node.getName();
      String ptName = node.getPrimaryNodeType().getName();

      NodeImpl newNode;

      //Value[] mtValues = node.getProperty("jcr:mixinTypes").getValues();
      NodeType[] mixinTypes = node.getMixinNodeTypes();
      String[] mtNames = new String[mixinTypes.length];
      for(int i=0; i<mtNames.length; i++)
        mtNames[i] = mixinTypes[i].getName();
      newNode = curParent().createChildNode(name, ptName, mtNames, false);
      
      System.out.println("frozen child node>> "+node.getPath()+" "+node.getPrimaryNodeType().getName());

      if(node.isNodeType("mix:referenceable")) {
        
        // look for the same UUID
        try {
          NodeImpl corresp = (NodeImpl)context.getSession().getNodeByUUID(newNode.getUUID());
          if(!corresp.getLocation().isDescendantOf(context.getLocation(), false)) {
            if(removeExisting) {
              corresp.remove();
              corresp.save();
            } else {
              throw new ItemExistsException("Item with the same UUID as "+newNode.getPath()+" already exists and removeExisting = false "+corresp.getPath());
            }
          }
        } catch (ItemNotFoundException e) {
        } 
      } 
      
      parents.push(newNode);
    }
    
  }
  
  protected void leaving(Property property, int level)
      throws RepositoryException {
  }

  protected void leaving(Node node, int level) throws RepositoryException {
    parents.pop();
  }
  
  private NodeImpl curParent() {
    return (NodeImpl)parents.peek();
  }
  
  
  private class PropertyInfo {
    
    private Property property;
    
    private PropertyInfo(Property property) {
      this.property = property;
    }
    
    private String getName() throws RepositoryException {
      String name = property.getName();
      if(property.getParent().isNodeType("nt:frozenNode")) {  
        if(name.equals("jcr:primaryType") ||
          name.equals("jcr:mixinTypes") || 
          name.equals("jcr:uuid") ||
          name.equals("jcr:frozenPrimaryType") ||
          name.equals("jcr:frozenMixinTypes") ||
          name.equals("jcr:frozenUuid")) {
          //ignore as it belongs to frozen
          
        } else {
          // normal property - add it here
          curParent().createChildProperty(name, 
              ((PropertyImpl)property).getValueArray(),
              property.getType());
        }
        return null;
        
      } else if(name.equals("jcr:childVersionHistory")) {
        return null;
      }

      return name;
      
    }
    
    private Value[] getValues() throws RepositoryException {
      return ((PropertyImpl)property).getValueArray();
    }
    
    private int getType() throws RepositoryException {
      return property.getType();
    }

  }
}
