/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.importing;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;

import javax.jcr.ImportUUIDBehavior;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.PropertyDefinition;

import org.apache.commons.digester.Rule;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.value.ReferenceValue;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * Created by The eXo Platform SARL
 *  Common class for Digester Rules
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: CommonRule.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class CommonRule extends Rule {
  protected final Log log = LogFactory.getLog(this.getClass());

  private int uuidBehavior = ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW;

  //key -- old uuid,
  //value -- new uuid
  private Map mapOldUuid2NewOne = new java.util.Hashtable();

  //key -- old id
  //value -- value holder collection
  private Map storedReferenceProperties = new java.util.Hashtable();

  /**
   * @see   javax.jcr.ImportUUIDBehavior
   */
  public int getUuidBehavior() {
    return uuidBehavior;
  }

  /**
   * @see   javax.jcr.ImportUUIDBehavior
   */
  public void setUuidBehavior(int uuidBehavior) {
    this.uuidBehavior = uuidBehavior;
  }

  //stack of nodes, topmost is current
  private Stack nodesStack = new Stack();

  protected Session getSession() {
    throw new NullPointerException("Session is not specified");
  }

  /**
   * stack of nodes, topmost is current
   */
  protected Stack getNodesStack() {
    return nodesStack;
  }

  final protected void pushNode(Node node) {
    getNodesStack().push(node);
  }

  final protected Node popNode() {
    //        return peekNode();
    return (Node) getNodesStack().pop();
  }

  final protected Node peekNode() {
    return (Node) getNodesStack().peek();
  }

  public void begin(String namespace, String name, Attributes attributes)
      throws Exception {
    log.debug("begin() name [" + name + "] " + attsSnapshot(attributes)
        + " namespace[" + namespace + "]");
  }

  public void body(String namespace, String name, String text) throws Exception {

    log.debug("body() name[" + name + "] text[" + text + "] namespace ["
        + namespace + "]");
  }

  public void end(String namespace, String name) throws Exception {

    log.debug("end() name[" + name + "] namespace [" + namespace + "]");

  }

  protected String attsSnapshot(Attributes attributes) {
    String strAtts = "atts count [ " + attributes.getLength() + "] (";
    for (int i = 0; i < attributes.getLength(); i++) {
      strAtts += " (qName[" + attributes.getQName(i) + "] value["
          + attributes.getValue(attributes.getQName(i)) + "]);";
    }
    strAtts += " )";
    return strAtts;
  }

  protected Node createNode(String nodeNamespaceURI, String nodeLocalName,
      String primaryNodeTypeName) throws RepositoryException, SAXException {

    //default primaryNodeType is "nt:unstructured"
    if (primaryNodeTypeName == null)
      primaryNodeTypeName = "nt:unstructured";

    String nodeQName = nodeLocalName;
    if (!nodeNamespaceURI.equals("")) {
      nodeQName = getSession().getNamespacePrefix(nodeNamespaceURI) + ":"
          + nodeQName;
    }
    return createNode(nodeQName, primaryNodeTypeName);

  }

  private void skipProperty(String propertyName) {
    log.debug("Skip property name[" + propertyName + "]");
    // propertyValue["+ propertyValue+"]");
  }

  private PropertyDefinition getPropertyDefinition(String propertyName)
      throws RepositoryException {
    Node node = peekNode();
    PropertyDefinition[] propDefinitions = node.getPrimaryNodeType()
        .getPropertyDefinitions();

    //with getName == "*"
    PropertyDefinition asterixPropDefinitions = null;
    //with getName == propertyName
    PropertyDefinition requisitePropDefinitions = null;

    int i = 0;
    while (i < propDefinitions.length) {
      if (propDefinitions[i].getName().equals(propertyName)) {
        requisitePropDefinitions = propDefinitions[i];
        log.trace("Find property def for [" + propertyName + "]");
        break;
      }

      if (propDefinitions[i].getName().equals("*")) {
        asterixPropDefinitions = propDefinitions[i];
        log.trace("Find a definition for a residual set of properties");
      }
      i++;
    }

    if (requisitePropDefinitions == null) {
      requisitePropDefinitions = asterixPropDefinitions;
      log.trace("No def found for property [" + propertyName
          + "] Use a definition for '*' (if exists)");
    }
    return requisitePropDefinitions;
  }

  /**
   * Return true if property whould be skipped
   * @todo check protected?!
   */

  private boolean isSkipProperty(String propertyName)
      throws RepositoryException {
    PropertyDefinition propDef = getPropertyDefinition(propertyName);

    //no def found
    if (propDef == null) {
      log.trace("No def found for property [" + propertyName + "]");
      return true;
    }

    //auto created
    if (propDef.isAutoCreated()) {
      log.trace("Property [" + propertyName + "] is Autocreated");
      //           return true;
    }

    //protected
    if (propDef.isProtected()) {
      log.trace("Property [" + propertyName + "] is Protected");
      return true;
    }
    return false;
  }

  //specific behaviour
  private boolean isRespectProperty(String propertyQName, int type)
      throws RepositoryException {
    boolean result = false;
    Node node = peekNode();

    //jcr:uuid
    result |= ("jcr:uuid".equals(propertyQName) && node
        .isNodeType("mix:referenceable"));

    //reference
    result |= type == PropertyType.REFERENCE;

    return result;
  }

  private Property respectProperty(String propertyQName, Value[] values,
      int type, boolean multivalued) throws RepositoryException, SAXException {

    log.trace("respected property [" + propertyQName + "]");
    //jcr:uuid

    if ("jcr:uuid".equals(propertyQName)) {
      Property prop = createJcrUuidProperty(values[0].getString());
      //map old uuid to new one
      if (prop != null) {
        String oldUuid = values[0].getString();
        log.debug("Map old uuid [" + oldUuid + "] to new one ["
            + prop.getString() + "]");
        mapOldUuid2NewOne.put(oldUuid, prop.getString());
        rollStoredReferences(oldUuid);
      }
      return prop;
    }

    if (type == PropertyType.REFERENCE)
      return createReferenceProperty(propertyQName, values, multivalued);

    return null;
  }

  private Collection getStoredReferenceProperties(String oldUuid) {
    if (!storedReferenceProperties.containsKey(oldUuid)) {
      storedReferenceProperties.put(oldUuid, new ArrayList());
    }
    return (Collection) storedReferenceProperties.get(oldUuid);

  }

  private boolean isOldUuid2NewOneMapped(Value[] values)
      throws RepositoryException {
    boolean result = true;
    for (int i = 0; i < values.length && result; i++) {
      result &= mapOldUuid2NewOne.containsKey(values[i].getString());
    }
    return result;
  }

  private Value[] remapOldUuid2NewOne(Value[] values)
      throws RepositoryException {
    Value[] newValuesArray = new Value[values.length];
    for (int i = 0; i < newValuesArray.length; i++) {
      String oldUuid = values[i].getString();
      newValuesArray[i] = new ReferenceValue((String) mapOldUuid2NewOne
          .get(oldUuid));
    }
    return newValuesArray;
  }

  private void rollStoredReferences(String oldUuid) throws RepositoryException,
      SAXException {

    Iterator refProperties = new ArrayList(
        getStoredReferenceProperties(oldUuid)).iterator();

    if (refProperties.hasNext())
      log.debug("Roll Stored References Properties for old uuid[" + oldUuid
          + "]");

    while (refProperties.hasNext()) {
      ValueHolder holder = (ValueHolder) refProperties.next();
      if (isOldUuid2NewOneMapped(holder.values)) {
        log.trace("All values mapped to new uuids. Create property name["
            + holder.propertyQName + "] for node parentNode ["
            + holder.parentNode.getPath() + "]");

        if (holder.multivalued)
          holder.parentNode.setProperty(holder.propertyQName,
              remapOldUuid2NewOne(holder.values), holder.type);
        else
          holder.parentNode.setProperty(holder.propertyQName,
              remapOldUuid2NewOne(holder.values)[0], holder.type);
        //drop stored the holder
        for (int i = 0; i < holder.values.length; i++) {
          Collection holders = getStoredReferenceProperties(holder.values[i]
              .getString());
          //                    log.trace("Remove stored reference value ["+holder.values[i].getString());
          holders.remove(holder);
        }
      } else {
        log.trace("Not all values mapped to new uuids. wait again ....");
      }
    }

  }

  /**
   * Store property data with references to unmapped nodes (referensable)
   * till node is mapped
   *
   */
  private void storeReferenceProperty(String propertyQName, Value[] values,
      boolean multivalued) throws RepositoryException {
    int i = 0;
    while (i < values.length) {
      String oldUuid = values[i].getString();

      boolean mappingExists = isOldUuid2NewOneMapped(new Value[] { values[i] });

      if (!mappingExists) {
        log.debug("Mapping for uuid[" + oldUuid
            + "] not exists yet. Store reference till mapping find");
        ValueHolder valueHolder = new ValueHolder(peekNode(), propertyQName,
            values, PropertyType.REFERENCE, multivalued);

        getStoredReferenceProperties(oldUuid).add(valueHolder);
      }
      i++;
    }

  }

  private Property createReferenceProperty(String propertyQName,
      Value[] values, boolean multivalued) throws RepositoryException,
      SAXException {

    Property resultProperty = null;

    //all uuids mapped -- create property
    if (isOldUuid2NewOneMapped(values)) {
      log.debug("all uuids for referenseble property [" + propertyQName
          + "] mapped. Create property ");
      if (multivalued)
        resultProperty = peekNode().setProperty(propertyQName,
            remapOldUuid2NewOne(values), PropertyType.REFERENCE);
      else
        resultProperty = peekNode().setProperty(propertyQName,
            remapOldUuid2NewOne(values)[0], PropertyType.REFERENCE);
    } else {
      storeReferenceProperty(propertyQName, values, multivalued);
    }
    return resultProperty;
  }

  private Property createJcrUuidProperty(String uuid)
      throws RepositoryException, SAXException {
    Node node = peekNode();
    Value value = new ReferenceValue(uuid);
    Property prop = null;
    log.trace("createJcrUuidProperty uuid[" + uuid + "] uuidBehaviour ["
        + getUuidBehavior() + "]");

    switch (getUuidBehavior()) {
    //IMPORT_UUID_CREATE_NEW
    case ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW:
      log.debug("Return autocreated property with genereted uuid");
      prop = node.getProperty("jcr:uuid");
      break;

    //IMPORT_UUID_COLLISION_THROW
    case ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW:
      try {
        Node sameUuidNode = getSession().getNodeByUUID(uuid);
        throw new SAXException("Node with property jcr:uuid [" + uuid
            + "] already exists [" + sameUuidNode.getPath() + "]");

      } catch (ItemNotFoundException ex) {
        log.trace("Item with uuid [" + uuid + "] not found");
        NodeImpl nodeImpl = (NodeImpl) node;

        prop = createProtectedProperty(node, "jcr:uuid", new Value[] { value },
            PropertyType.STRING, false);
        log
            .trace("Created uuid property [" + prop.getValue().getString()
                + "]");
        break;

      }

    //IMPORT_UUID_COLLISION_REMOVE_EXISTING
    case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING:
      try {
        Node sameUuidNode = getSession().getNodeByUUID(uuid);
        log.debug("Node with uuid [" + uuid + "] is found. Will be deleted.");
        sameUuidNode.remove();
      } catch (ItemNotFoundException ex) {
        log
            .trace("Item with uuid [" + uuid
                + "] not found. Nothing to remove.");
      }

      prop = createProtectedProperty(node, "jcr:uuid", new Value[] { value },
          PropertyType.STRING, false);
      log.trace("Created uuid property [" + prop.getValue().getString() + "]");
      break;

    //IMPORT_UUID_COLLISION_REPLACE_EXISTING
    case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING:
      try {
        Node sameUuidNode = getSession().getNodeByUUID(uuid);
        log.debug("Node with uuid [" + uuid
            + "] is found. Will be replaced. path [" + sameUuidNode.getPath()
            + "]");
        log.debug("sameUuidNode jcr:uuid ["
            + sameUuidNode.getProperty("jcr:uuid").getString() + "]");

        if (uuid.equals(sameUuidNode.getProperty("jcr:uuid").getString())) {

          //                        String destPath = sameUuidNode.getParent().getPath()+"/"+node.getName();
          String destPath = sameUuidNode.getParent().getPath();
          String nodeName = node.getName();

          log.trace("New src path [" + node.getPath() + "] dest Path ["
              + destPath + "]");
          sameUuidNode.remove();
          getSession().move(node.getPath(), destPath);
          popNode();

          //todo: it will be troubles with same name siblings
          pushNode((Node) getSession().getItem(destPath + "/" + nodeName));

        }

      } catch (ItemNotFoundException ex) {
        log.trace("Item with uuid [" + uuid
            + "] not found. Nothing to replace.");
      }

      log.trace("Current node path [" + peekNode().getPath() + "]");
      prop = createProtectedProperty(peekNode(), "jcr:uuid",
          new Value[] { value }, PropertyType.STRING, false);
      log.trace("Created uuid property [" + prop.getValue().getString() + "]");
      break;

    default:
      throw new RuntimeException("Uuid behavior not supported ["
          + getUuidBehavior() + "]");
    }
    return prop;

  }

  private Property createProtectedProperty(Node node, String propertyQName,
      Value[] values, int type, boolean multivalued)
      throws RepositoryException, SAXException {
    NodeImpl nodeImpl = (NodeImpl) node;
    return nodeImpl.createChildProperty(propertyQName, values, type);
    //        return nodeImpl.initProperty(propertyQName,values,type,multivalued,null);
  }

  private Property createPropertyInternal(String propertyQName, Value[] values,
      int type, boolean multivalued) throws RepositoryException, SAXException {

    //respect property 7.3.3
    if (isRespectProperty(propertyQName, type)) {
      return respectProperty(propertyQName, values, type, multivalued);
    } else
    //skip property 7.3.3
    if (isSkipProperty(propertyQName)) {
      skipProperty(propertyQName);
      return null;
    } else {
      Node node = peekNode();
      //default
      log
          .debug("createPropertyInternal propertyQName [" + propertyQName
              + "] values count [" + values + "] multivalued [" + multivalued
              + "]");

      if (multivalued)
        return node.setProperty(propertyQName, values, type);
      else
        return node.setProperty(propertyQName, values[0], type);
    }

  }

  protected Property createProperty(String propertyQName, Value value, int type)
      throws RepositoryException, SAXException {
    boolean multivalued = false;
    PropertyDefinition propDef = getPropertyDefinition(propertyQName);

    //set multivalued to true if condition is true
    //?
    if (propDef != null && !propDef.getName().equals("*")
        && propDef.isMultiple()) {
      log.debug("Seems property is multivalued [" + propertyQName + "]");
      multivalued = true;
    }

    return createPropertyInternal(propertyQName, new Value[] { value }, type,
        multivalued);
  }

  protected Property createProperty(String propertyQName, Value[] values,
      int type) throws RepositoryException, SAXException {

    return createPropertyInternal(propertyQName, values, type, true);
  }

  protected Node createNode(String nodeQName, String primaryNodeTypeName)
      throws RepositoryException, SAXException {
    //        return null;
    Node parentNode = peekNode();

    log.debug("create node [" + nodeQName + "] with primaryNodeTypeName["
        + primaryNodeTypeName + "] parent node name[" + parentNode.getName()
        + "]");

    try {
      parentNode.getProperty(nodeQName);
      throw new ConstraintViolationException("Parent node ["
          + parentNode.getPath() + "] have property [" + nodeQName
          + "], you can't create cahild node with name [" + nodeQName + "]");

    } catch (PathNotFoundException ex) {
      Node newNode = parentNode.addNode(nodeQName, primaryNodeTypeName);

      pushNode(newNode);
      return newNode;
    }
  }

}

class ValueHolder {
  ValueHolder(Node parentNode, String propertyQName, Value[] values, int type,
      boolean multivalued) {
    this.parentNode = parentNode;
    this.propertyQName = propertyQName;
    this.values = values;
    this.type = type;
    this.multivalued = multivalued;
  }

  public Node parentNode;

  public String propertyQName;

  public Value[] values;

  public int type;

  public boolean multivalued;
}