/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.importing;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import java.util.Stack;

/**
 * Created by The eXo Platform SARL
 *  Used as rule vor sv:value element in org.apache.commons.digester
 *  on import repository content from System view (JSR-170 7.3.1)
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: SysViewValueRule.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class SysViewValueRule extends CommonSysViewRule {

    SysViewValueRule(Stack nodesStack){
        super(nodesStack);
    }


    private void checkIfExistsParentProperty() throws SAXException{
            XMLElementDefinition parentElementDef = peekElementDef();
            try {
                checkCurrentElementDefinition(javax.jcr.Property.class);
            } catch (SAXException ex) {
                String errorText = parentElementDef == null ? "" :
                                   " (Parent [" + parentElementDef.toString() +
                                   "])";
                errorText = "Document not valid, no found parent property." +
                            errorText;
                log.error(errorText);
                throw new SAXException(errorText);
            }
    }

    public void begin(String namespace, String name, Attributes attributes) throws
            Exception {
        super.begin(namespace, name, attributes);

        checkIfExistsParentProperty();

        XMLElementDefinition elementDef =
                newElementDefinition(namespace, name, attributes,
                                     javax.jcr.Value.class);
        peekElementDef().addChild(elementDef);
        digester.push(elementDef);
    }

    public void body(String namespace, String name, String text) throws
            Exception {
        super.body(namespace, name, text);
        checkCurrentElementDefinition(namespace, name, javax.jcr.Value.class);
        peekElementDef().setBody(text);
    }

    public void end(String namespace, String name) throws Exception {
        super.end(namespace, name);
        checkCurrentElementDefinition(namespace, name, javax.jcr.Value.class);
        digester.pop();
    }

}
