/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.rdb.container;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.InvalidItemStateException;
import javax.jcr.ItemExistsException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.ItemRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBItemRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBPropertyRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.NodeRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.PropertyRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.SingleRDBPropertyRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.ValueRecord;
import org.exoplatform.services.log.LogService;
import org.hibernate.NonUniqueObjectException;
import org.hibernate.Session;



/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: MultiRDBWorkspaceContainerImpl.java 3547 2006-01-03 08:42:56Z geaz $
 */
public class MultiRDBWorkspaceContainerImpl extends RDBWorkspaceContainerImpl {

  protected static String FIND_ITEM_BY_PATH = "from MultiRDBItemRecord as item where item.path=?";

  protected static String FIND_NODE_BY_PATH = "from MultiRDBNodeRecord as item where item.path=?";

  protected static String FIND_PROPERTY_BY_PATH = "from MultiRDBPropertyRecord as item where item.path=?";

  protected static String FIND_PROPS_BY_TYPE_AND_VALUE = "select distinct prop from MultiRDBValueRecord as val, MultiRDBPropertyRecord as prop inner join val.property where prop.valuesType=? and val.data=?";
  
  protected static String FIND_PROPS_BY_VALUE = "from MultiRDBValueRecord as val where val.data=?";

  public MultiRDBWorkspaceContainerImpl(
      WorkspaceEntry wsConfig,
  //HibernateConfigurationsService hibernateContainer,
      LogService logService) throws RepositoryException,
      RepositoryConfigurationException {
   
    super(wsConfig, logService);
    log.info("MultiRDBWorkspacecontainerImpl (cEntry)" + name
        + " hibernate container =" + srcName);

  }

  synchronized protected NodeRecord addNodeRecord(NodeData data,
      Session hibernateSession) throws RepositoryException,
      ItemExistsException, InvalidItemStateException {

    log.debug("RDB Container is adding Node: " + data.getQPath().getAsString()
        + " index " + data.getQPath().getIndex() + " UUID: " + data.getUUID());

    NodeRecord parent = findParentNodeRecord(data);
    MultiRDBNodeRecord nodeRecord = new MultiRDBNodeRecord();
    nodeRecord.setId(data.getUUID());
    nodeRecord.setPath(data.getQPath().getAsString());
    nodeRecord.setParentNode(parent);
    nodeRecord.setOrderNumber(data.getOrderNumber());

    if (parent != null) // not root
      parent.addChildNode(nodeRecord);

    try {
      hibernateSession.persist(nodeRecord);
    } catch (NonUniqueObjectException e) {
      throw new ItemExistsException("Item with UUID "+data.getUUID()+" already exists");
    }


    return nodeRecord;

  }

  synchronized protected PropertyRecord addPropertyRecord(PropertyData prop,
      NodeRecord parent, Session hibernateSession)
      throws RepositoryException, ItemExistsException {

    MultiRDBPropertyRecord propertyRecord = new MultiRDBPropertyRecord();
    propertyRecord.setId(prop.getUUID());
    propertyRecord.setPath(prop.getQPath().getAsString());
    propertyRecord.setType(prop.getType());
    propertyRecord.setValues(prop.getValues());

    parent.addProperty(propertyRecord);

    log.debug("RDB Container is adding " + propertyRecord);

    hibernateSession.persist(propertyRecord);

    return propertyRecord;

  }

  protected ItemRecord findItemByPath(String path, Session hibernateSession) {
    return (MultiRDBItemRecord) hibernateSession.createQuery(FIND_ITEM_BY_PATH)
        .setString(0, path).uniqueResult();
  }

  protected ItemRecord findItemByUUID(String uuid, Session hibernateSession) {
    return (MultiRDBItemRecord) hibernateSession.get(MultiRDBItemRecord.class,
        uuid);
  }

  protected NodeRecord findNodeByPath(String path, Session hibernateSession) {
    return (MultiRDBNodeRecord) hibernateSession.createQuery(FIND_NODE_BY_PATH)
        .setString(0, path).uniqueResult();
  }

  protected NodeRecord findNodeByUUID(String uuid, Session hibernateSession) {
    return (MultiRDBNodeRecord) hibernateSession.get(MultiRDBNodeRecord.class,
        uuid);
  }

  protected PropertyRecord findPropertyByPath(String path,
      Session hibernateSession) {
    return (MultiRDBPropertyRecord) hibernateSession.createQuery(
        FIND_PROPERTY_BY_PATH).setString(0, path).uniqueResult();
  }

  protected PropertyRecord findPropertyByUUID(String uuid,
      Session hibernateSession) {
    return (MultiRDBPropertyRecord) hibernateSession.get(
        MultiRDBPropertyRecord.class, uuid);
  }

  protected List findReferences(String uuid, Session hibernateSession) {
    //return hibernateSession.createQuery(FIND_PROPS_BY_TYPE_AND_VALUE)
    //    .setInteger(0, PropertyType.REFERENCE).setBinary(1, uuid.getBytes())
    //    .list();
    List res1 = hibernateSession.createQuery(FIND_PROPS_BY_VALUE).setBinary(0,
        uuid.getBytes()).list();
    List props = new ArrayList();
    for (int i = 0; i < res1.size(); i++) {
      ValueRecord rec = (ValueRecord) res1.get(i);
      if (rec.getProperty().getType() == PropertyType.REFERENCE) {
        props.add(rec.getProperty());
      }
    }

    return props;

  }

  public boolean isMultiplicityAllowed() {
    return false;
  }

}