/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.storage.rdb.container.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.datamodel.InternalQName;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: MultiRDBNodeRecord.java 3510 2005-12-29 17:32:57Z geaz $
 * 
 * @hibernate.joined-subclass table="JCR_MNODE" dynamic-update="true"
 * @hibernate.joined-subclass-key column="ID"
 * @hibernate.collection-index column="ORDER_NUM"
 * @hibe---rnate.cache usage="read-write"
 */

public class MultiRDBNodeRecord extends MultiRDBItemRecord implements NodeRecord {

  protected int orderNum;

  protected Set properties = new HashSet();

  protected Set nodes = new HashSet();

  protected InternalQName primaryType;

  protected InternalQName[] mixinTypes;

  protected AccessControlList acl;

  public MultiRDBNodeRecord() {
  }

  /**
   * --lazy="false"--
   * 
   * @hibernate.set cascade="all" inverse="true"
   * 
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBPropertyRecord"
   * @hibernate.collection-key column="PARENT_ID"
   *  
   */
  public Set getProperties() {
    return properties;
  }

  public List getChildProperties() throws RepositoryException {
    ArrayList props = null;
    if (properties != null)
      props = new ArrayList(properties);
    return props;

  }

  /**
   * @param properties
   */
  public void setProperties(Set properties) {
    this.properties = properties;
  }

  public void addProperty(PropertyRecord property) {
    property.setParentNode(this);
    if (properties == null)
      properties = new HashSet();
    this.properties.add(property);
  }

  /**
   * ---lazy="proxy"---
   * 
   * @hibernate.set cascade="all" inverse="true"
   * 
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord"
   * @hibernate.collection-key column="PARENT_ID"
   */
  public Set getNodes() {
    return nodes;
  }

  public void setNodes(Set childNodes) {
    this.nodes = childNodes;
  }

  public void addChildNode(NodeRecord node) {
    node.setParentNode(this);
    if (nodes == null)
      nodes = new HashSet();
    this.nodes.add(node);
  }

  public List getChildNodes() throws IllegalStateException, RepositoryException {
    ArrayList nodeList = null;
    if (nodes != null)
      nodeList = new ArrayList(nodes);

    return nodeList;
  }

  public InternalQName getPrimaryTypeName() {
    return primaryType;
  }

  public AccessControlList getChildrenACL() {
    return acl;
  }

  public InternalQName[] getMixinTypeNames() {
    return mixinTypes;
  }

  public void setPrimaryType(InternalQName primaryType) {
    this.primaryType = primaryType;
  }

  public void setMixinTypes(InternalQName[] mixinTypes) {
    this.mixinTypes = mixinTypes;
  }

  public void setChildrenACL(AccessControlList acl) {
    this.acl = acl;
  }

  /**
   * @hibernate.property column="ORDER_NUM"
   */
  public int getOrderNumber() {
    return orderNum;
  }

  public void setOrderNumber(int orderNum) {
    this.orderNum = orderNum;
  }

  /**
   * @hibernate.many-to-one column="PARENT_ID"
   *                        class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord"
   */
  public NodeRecord getParentNode() {
    return parentNode;
  }

  public void setParentNode(NodeRecord parent) {
    this.parentNode = parent;
    if(parent != null)
      this.parentUUID = new String(parent.getId());
  }
  	
  public boolean isNode() {
    return true;
  }

}