/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.xml;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import javax.jcr.ImportUUIDBehavior;
import javax.jcr.InvalidSerializedDataException;
import javax.jcr.NamespaceException;
import javax.jcr.NamespaceRegistry;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeType;
import javax.xml.parsers.SAXParserFactory;

import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.util.ISO9075;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: DocNodeImporter.java 3607 2006-01-04 11:28:33Z peterit $
 */

public class DocNodeImporter extends NodeImporter implements ContentHandler, ErrorHandler {

  private SessionImpl       session;

  private NodeImpl          rootNode;

  // private NodeImpl parent;
  private NamespaceRegistry nsReg;

  private Stack             tree;

  private int               uuidBehavior    = -1;

  private PrintStream       userErrorStream = null;

  public DocNodeImporter(NodeImpl parent) throws RepositoryException {
    super(parent);
    tree = new Stack();
    session = (SessionImpl) parent.getSession();
    // this.parent = parent;
    this.nsReg = session.getWorkspace().getNamespaceRegistry();
    tree.push(parent);
  }

  public NodeImpl parse(InputStream stream) throws IOException, SAXException {
    XMLReader reader;
    try {
      SAXParserFactory factory = SAXParserFactory.newInstance();
      factory.setNamespaceAware(true);
      reader = factory.newSAXParser().getXMLReader();

    } catch (Exception e) {
      throw new SAXException(e.getMessage());
    }
    reader.setContentHandler(this);
    reader.parse(new InputSource(stream));

    return rootNode;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.impl.xml.NodeImporter#startElement(java.lang.String,
   *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
   */
  public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
      throws SAXException {

    if ("jcr:root".equals(qName))
      qName = "";

    HashMap<String, String> props = new HashMap<String, String>();
    // default value of primary type
    String primaryNodeType = "nt:unstructured";
    props.put("jcr:primaryType", primaryNodeType);

    String[] mixinNodeTypes = null;

    if (atts != null) {
      int length = atts.getLength();
      for (int i = 0; i < length; i++) {
        String propName;
        String attrQName = atts.getQName(i);
        if (!"".equals(attrQName))
          propName = attrQName;
        else
          propName = atts.getLocalName(i);

        if (propName.equals("jcr:primaryType")) {
          primaryNodeType = atts.getValue(i);

          props.put(propName, atts.getValue(i));
        } else if (propName.equals("jcr:mixinTypes")) {
          mixinNodeTypes = atts.getValue(i).split(" ");
          for (int mi = 0; mi < mixinNodeTypes.length; mi++) {
            mixinNodeTypes[mi] = ISO9075.decode(mixinNodeTypes[mi]);
          }

          props.put(propName, atts.getValue(i));
        } else if (propName.equals("jcr:uuid")) {
          props.put(propName, atts.getValue(i));
        } else {
          props.put(propName, atts.getValue(i));
        }
      }
    }

    if (mixinNodeTypes == null) {
      mixinNodeTypes = new String[0];
    }

    // fill mixinNodeTypes array
    boolean hasMixReferenceable = false;
    for (int i = 0; i < mixinNodeTypes.length; i++) {
      if ("mix:referenceable".equals(mixinNodeTypes[i])) {
        hasMixReferenceable = true;
      }
    }

    NodeImpl node = null;
    Node sameUuidNode = null;
    String uuid = props.get("jcr:uuid");
    try {

      NodeImpl parentNode = parent();

      //System.err.println("========= node: " + qName + ", parent: " + parentNode.getPath());

      if (hasMixReferenceable && uuid != null) {

        //System.err.println("========= node: refeenceable");

        // Check for mix:referenceable node with uuid attr
        sameUuidNode = session.getNodeByUUID(uuid);
        if (sameUuidNode != null) {
          switch (uuidBehavior) {
          case ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW:
            // Incoming referenceable nodes are assigned newly created UUIDs
            // upon addition to the workspace. As a result UUID collisions
            // never occur.

            // reset UUID and it will be autocreated in session
            // System.err.println("===== IMPORT_UUID_CREATE_NEW");
            uuid = null;
            break;
          case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING:
            // If an incoming referenceable node has the same UUID as a node
            // already existing in the workspace then the already existing
            // node (and its subtree) is removed from wherever it may be in
            // the workspace before the incoming node is added. Note that this
            // can result in nodes �disappearing� from locations in the
            // workspace that are remote from the location to which the
            // incoming subtree is being written.

            // System.err.println("=====
            // IMPORT_UUID_COLLISION_REMOVE_EXISTING");
            NodeIterator samePatterns = sameUuidNode.getNodes(parentNode.getName());
            if (samePatterns.hasNext()) {
              // Throw exception
              throw new InvalidSerializedDataException(
                  "A uuidBehavior is set to IMPORT_UUID_COLLISION_REMOVE_EXISTING and an incoming node has the same UUID as the node at parentAbsPath or one of its ancestors");
            } else {
              // remove node
              // Node sameUuidParent = sameUuidNode.getParent();
              sameUuidNode.remove();
              sameUuidNode.save();
              sameUuidNode = null;
              // sameUuidParent.save();
            }
            break;
          case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING:
            // If an incoming referenceable node has the same UUID as a node
            // already existing in the workspace, then the already existing
            // node is replaced by the incoming node in the same position as
            // the existing node. Note that this may result in the incoming
            // subtree being disaggregated and �spread around� to different
            // locations in the workspace. In the most extreme case this
            // behavior may result in no node at all being added as child of
            // parentAbsPath. This will occur if the topmost element of the
            // incoming XML has the same UUID as an existing node elsewhere in
            // the workspace.

            // replace in same location
            // System.err
            // .println("===== IMPORT_UUID_COLLISION_REPLACE_EXISTING");
            parentNode = (NodeImpl) sameUuidNode.getParent();
            sameUuidNode.remove();
            sameUuidNode.save();
            sameUuidNode = null;
            break;
          case ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW:
            // If an incoming referenceable node has the same UUID as a node
            // already existing in the workspace then a SAXException is thrown
            // by the ContentHandler during deserialization.
            throw new InvalidSerializedDataException(
                "An incoming referenceable node has the same UUID as a node already existing in the workspace!");
          default:
          }
        }
      } else {
        // other cases with same names etc.

        //System.err.println("========= node: pri_type: " + props.get("jcr:primaryType"));

        try {
          // Check for node with this name in session
          NodeImpl sameNameNode = (NodeImpl) parentNode.getNode(qName);
          if (!sameNameNode.getDefinition().allowsSameNameSiblings()) {
            // Throw exception
            String msg = "A node with this name (" + qName + ") is already exists. "
                + "Overwriting an existing node during import is impossible.";
            //System.err.println("========= node found: " + msg);
            throw new SAXException(msg);
          }
        } catch (PathNotFoundException e) {
          // ok
          //System.err.println("========= node not found: " + qName);
        }
      }

      // Code before 20.12.2005, Peter Nedonosko
      // node = parent().createChildNode(qName, primaryNodeType, mixinNodeTypes,
      // false); // addAutoCreated=false

      node = parentNode.createChildNode(qName, primaryNodeType, mixinNodeTypes, uuid, true, false); // autoCreated=true,
                                                                                                    // addAutoCreated=false

      Iterator keys = props.keySet().iterator();
      while (keys.hasNext()) {
        String key = (String) keys.next();

        if (key.equals("jcr:primaryType")) {
          Value value = session.getValueFactory().createValue((String) props.get(key),
              PropertyType.NAME);
          node.createChildProperty(key, new Value[] { value }, PropertyType.NAME);
        } else if (key.equals("jcr:mixinTypes")) {

          Value[] values = new Value[mixinNodeTypes.length];
          for (int vi = 0; vi < mixinNodeTypes.length; vi++) {
            Value value = session.getValueFactory().createValue((String) props.get(key));
            values[vi] = value;
          }
          node.createChildProperty(key, values, PropertyType.NAME);
        } else if (key.equals("jcr:uuid")) {
          // reference value!
          Value value = session.getValueFactory().createValue((String) props.get(key));
          node.createChildProperty(key, new Value[] { value }, PropertyType.REFERENCE);
        } else {
          // string value!

          Value value = session.getValueFactory().createValue((String) props.get(key));
          node.createChildProperty(key, new Value[] { value }, PropertyType.STRING);
        }
      }

      // Save all changes for this item
      // session.save();

    } catch (Exception e) {
      e.printStackTrace();
      throw new SAXException(e.getMessage(), e);
    }
    if (rootNode == null) {
      rootNode = node;
    }

    tree.push(node);
  }

  public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
    tree.pop();
  }

  public void characters(char[] ch, int start, int length) throws SAXException {
    // System.out.println("chars "+new String(ch)+" "+length+" "+start);
    StringBuffer text = new StringBuffer();
    text.append(ch, start, length);

    try {
      NodeImpl txtNode = parent().createChildNode("jcr:xmltext", "nt:unstructured", new String[0],
          false);
      txtNode.createChildProperty("jcr:primaryType", new Value[] { session.getValueFactory()
          .createValue("nt:unstructured", PropertyType.NAME) }, PropertyType.NAME);
      txtNode.createChildProperty("jcr:xmlcharacters", new Value[] { session.getValueFactory()
          .createValue(new String(text), PropertyType.STRING) }, PropertyType.STRING);
    } catch (RepositoryException e) {
      throw new RuntimeException(e);
    }
  }

  public void setDocumentLocator(Locator locator) {
  }

  public void startDocument() throws SAXException {
  }

  public void endDocument() throws SAXException {
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {
    // System.out.println("Start prefix mapping "+prefix+" "+uri);
    try {
      nsReg.getPrefix(uri);
    } catch (NamespaceException e) {
      try {
        nsReg.registerNamespace(prefix, uri);
      } catch (NamespaceException e1) {
        throw new RuntimeException(e1);
      } catch (RepositoryException e1) {
        throw new RuntimeException(e1);
      }

    } catch (RepositoryException e) {
      throw new RuntimeException(e);
    }

  }

  public void endPrefixMapping(String prefix) throws SAXException {
  }

  public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
  }

  public void processingInstruction(String target, String data) throws SAXException {
  }

  public void skippedEntity(String name) throws SAXException {
  }

  private NodeImpl parent() {
    return (NodeImpl) tree.peek();
  }

  public void setUuidBehavior(int uuidBehavior) {
    this.uuidBehavior = uuidBehavior;
  }

  public int getUuidBehavior() {
    return uuidBehavior;
  }

  // ============ org.xml.sax.ErrorHandler implementation ============

  /*
   * (non-Javadoc)
   * 
   * @see org.xml.sax.ErrorHandler#error(org.xml.sax.SAXParseException)
   */
  public void error(SAXParseException err) throws SAXException {
    // TODO Auto-generated method stub
    errorErrorLog(null, err);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.xml.sax.ErrorHandler#fatalError(org.xml.sax.SAXParseException)
   */
  public void fatalError(SAXParseException err) throws SAXException {
    // TODO Auto-generated method stub
    fatalErrorLog(null, err);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.xml.sax.ErrorHandler#warning(org.xml.sax.SAXParseException)
   */
  public void warning(SAXParseException err) throws SAXException {
    // TODO Auto-generated method stub
    warnErrorLog(null, err);
  }

  private void warnErrorLog(String message, Throwable exception) {
    toErrorLog("[WARN] " + (message != null && message.length() > 0 ? message + "; " : ""),
        exception);
  }

  private void errorErrorLog(String message, Throwable exception) {
    toErrorLog("[ERROR] " + (message != null && message.length() > 0 ? message + "; " : ""),
        exception);
  }

  private void fatalErrorLog(String message, Throwable exception) {
    toErrorLog("[FATAL] " + (message != null && message.length() > 0 ? message + "; " : ""),
        exception);
  }

  private void toErrorLog(String message, Throwable exception) {

    // very simple impl, Peter Nedonosko
    String err = "ErrorHandler: " + (message != null && message.length() > 0 ? message + "; " : "")
        + "Exception: " + exception;
    PrintStream errStream = (userErrorStream != null ? userErrorStream : System.err);
    errStream.println(err);
    exception.printStackTrace(errStream);
  }

  public void setUserErrorStream(PrintStream printStream) {
    userErrorStream = printStream;
  }

  public PrintStream getUserErrorStream() {
    return userErrorStream;
  }
}
