/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.xml;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.codec.binary.Base64;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: SysNodeImporter.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class SysNodeImporter extends NodeImporter implements ContentHandler {

  private Stack tree;
  private String curPropName;
  private String curPropType;
  private ArrayList curPropValues;
//  private JCRPath curLocation;
  private List nodeInfos;
  private Session session;
  //private NodeImpl parent;
  private boolean rootNode;

  public SysNodeImporter(NodeImpl parent) /*throws RepositoryException*/ {
    super(parent);
    tree = new Stack();
    //this.parent = parent;
    nodeInfos = new ArrayList();
    this.session = parent.getSession();
    curPropValues = new ArrayList();
    rootNode = true;
  }
  
  public NodeImpl parse(InputStream stream) throws IOException, SAXException, RepositoryException {

  	// parse XML
    XMLReader reader;
    try {
      SAXParserFactory factory = SAXParserFactory.newInstance();
      factory.setNamespaceAware(true);
      reader = factory.newSAXParser().getXMLReader();
    } catch (Exception e) {
      throw new SAXException(e.getMessage());
    }
    reader.setContentHandler(this);
    reader.parse(new InputSource(stream));

    // build node
    NodeImpl node = null;
    
    for(int i=0; i<nodeInfos.size(); i++) {
        NodeInfo info = (NodeInfo) nodeInfos.get(i);
//    	System.out.println("Set items "+nodeInfos.size()+" "+parent.getPath()+" "+info.getRelPath());

    	NodeImpl n = parent.importChildNode(info.getRelPath(), info.getPrimaryTypeName(), 
    			info.getMixinTypeNames());
    	List props = info.getProperties();
    	for(int j=0; j<props.size(); j++) {
    		PropertyInfo prop = (PropertyInfo) props.get(j);
    		List valueList = prop.getValues();
    		Value[] values = new Value[valueList.size()];
        	for(int k=0; k<valueList.size(); k++) {
        		String val = new String((StringBuffer) valueList.get(k));
                if(prop.getType() == PropertyType.BINARY){
        			values[k] =  session.getValueFactory().createValue(
                            new ByteArrayInputStream(Base64.decodeBase64(val.getBytes())) );
                 } else {
        		    values[k] = session.getValueFactory().createValue(val, prop.getType());
                 }
        	}
    		n.createChildProperty(prop.getName(), values, prop.getType());
    	}
    	if(i==0)
    		node = n;
    }

    return node;

  }

  public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
    if (qName.equals("sv:node")) {
      try {
        String name = atts.getValue("sv:name");
        String relPath;
        if(!tree.isEmpty())
        	relPath = ((NodeInfo)tree.peek()).getRelPath()+"/"+name;
        else
        	relPath = name; 
      	NodeInfo info = new NodeInfo(relPath);
        tree.push(info);
        nodeInfos.add(info);
      } catch (Exception e) {
        e.printStackTrace();
        throw new SAXException(e.getMessage());
      }
    } else if (qName.equals("sv:property")) {
      //TODO manage UUID      
      try {
        curPropType = atts.getValue("sv:type");
        curPropName = atts.getValue("sv:name");
        curPropValues = new ArrayList();
      } catch (Exception e) {
        e.printStackTrace();
        throw new SAXException(e.getMessage(), e);
      }
    } else if (qName.equals("sv:value")) {
        // nothing to do
    } else
      throw new SAXException("'"+qName+"' is not allowed. Only sv:node, sv:property and sv:value are allowed");
  }

  public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
    if (qName.equals("sv:node")) {
//      nodeInfos.add(tree.pop());
        tree.pop();
    } else if (qName.equals("sv:property")) {
      ((NodeInfo)tree.peek()).addProperty(curPropName, curPropType, curPropValues);
    }
  }

  public void characters(char[] ch, int start, int length) throws SAXException {
     // property values
     StringBuffer curPropValue = new StringBuffer();
     curPropValue.append(ch, start, length);
     curPropValues.add(curPropValue);
  }

  public void setDocumentLocator(Locator locator) {
  }

  public void startDocument() throws SAXException {
  }

  public void endDocument() throws SAXException {
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {
  }

  public void endPrefixMapping(String prefix) throws SAXException {
  }

  public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
  }

  public void processingInstruction(String target, String data) throws SAXException {
  }

  public void skippedEntity(String name) throws SAXException {
  }

  private class NodeInfo {
    private String relPath;
    private ArrayList properties = new ArrayList();

    public NodeInfo(String relPath) {
        this.relPath = relPath;
    }

    public void addProperty(String name , String typeName, ArrayList strValues) {
        int type = ExtendedPropertyType.valueFromName(typeName);
  	    PropertyInfo prop = new PropertyInfo(name, type, strValues);
        properties.add(prop);
    }

    public ArrayList getProperties() {
        return properties;
    }

    public String getRelPath() {
        return relPath;
    }
    
    public String getPrimaryTypeName() throws RepositoryException {
        for(int i=0; i<properties.size(); i++) {
        	PropertyInfo prop = (PropertyInfo) properties.get(i);
        	if(prop.getName().equals("jcr:primaryType"))
        		return new String((StringBuffer)prop.getValues().get(0));
        }
        throw new RepositoryException("No primary type ");
    }
    
    public String[] getMixinTypeNames() {
    	ArrayList mixins = new ArrayList();
        for(int i=0; i<properties.size(); i++) {
        	PropertyInfo prop = (PropertyInfo) properties.get(i);
        	if(prop.getName().equals("jcr:mixinTypes")) {
        		for(int j=0; j<prop.getValues().size(); j++)
        		    mixins.add(new String((StringBuffer)prop.getValues().get(j)));
        	}
        }
        
        String[] mixinProps = new String[mixins.size()];
           for(int i=0; i<mixins.size(); i++) {
            	mixinProps[i] = (String) mixins.get(i);
           }
       	return mixinProps;
    }

  }
  
  private class PropertyInfo {
  	
        private String name;

		private int type;

		private List values;

		public PropertyInfo(String name, int type, List values) {
			super();
			this.name = name;
			this.type = type;
			this.values = values;
		}
  	

		/**
		 * @return Returns the name.
		 */
		public String getName() {
			return name;
		}
		/**
		 * @return Returns the type.
		 */
		public int getType() {
			return type;
		}
		/**
		 * @return Returns the values.
		 */
		public List getValues() {
			return values;
		}
  }  

}
