package org.exoplatform.services.jcr;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFactory;
import javax.jcr.Workspace;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.services.jcr.impl.core.CredentialsImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: BasePortalTest.java 3510 2005-12-29 17:32:57Z geaz $
 */
public abstract class BasePortalTest extends TestCase {
  protected Log log;

  protected static String TEMP_PATH = "./temp/fsroot";

  protected static String WORKSPACE = "ws";

  protected SessionImpl session;

  protected RepositoryImpl repository;

  protected CredentialsImpl credentials;

  //  protected ExoContainer servicesManager;
  protected PortalContainer servicesManager;

  protected Workspace workspace;

  protected RepositoryService repositoryService;

  protected LogService logService;

  protected Node root;

  protected ValueFactory valueFactory;

  public void setUp() throws Exception {

    logService = (LogService) RootContainer.getInstance()
        .getComponentInstanceOfType(LogService.class);

    log = logService.getLog("org.exoplatform.services.jcr");

    logService.setLogLevel("org.exoplatform.services.jcr", LogService.DEBUG,
        true);

    servicesManager = RootContainer.getInstance().getPortalContainer("exo");

    if (System.getProperty("java.security.auth.login.config") == null)
      System.setProperty("java.security.auth.login.config",
          "src/resources/login.conf");

    credentials = new CredentialsImpl("exo", "exo".toCharArray());

    repositoryService = (RepositoryService) servicesManager
        .getComponentInstanceOfType(RepositoryService.class);

    repository = (RepositoryImpl) repositoryService
        .getRepository(getRepositoryName());

    //repository.getContainer().start();

    if (!((RepositoryImpl) repository).isWorkspaceInitialized("ws"))
      ((RepositoryImpl) repository).initWorkspace("ws", "nt:unstructured");

    if (!((RepositoryImpl) repository).isWorkspaceInitialized("ws2"))
      ((RepositoryImpl) repository).initWorkspace("ws2", "nt:unstructured");

    session = (SessionImpl) repository.login(credentials, "ws");
    workspace = session.getWorkspace();
    root = session.getRootNode();
    valueFactory = session.getValueFactory();

    logService.setLogLevel("org.exoplatform.services.jcr", LogService.DEBUG,
        true);

    initRepository();

  }

  public void tearDown() throws Exception {
    //      log = logService.getLog("org.exoplatform.services.jcr");
    //      logService.setLogLevel("org.exoplatform.services.jcr", LogService.ERROR,
    // true);
  }

  protected abstract String getRepositoryName();

  public void initRepository() throws RepositoryException {
  }

}