package org.exoplatform.services.jcr.api.version;

import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.impl.core.CredentialsImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.itemvisitors.NodeDumpVisitor;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TestInitializingVersionHistory.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class TestInitializingVersionHistory extends JcrAPIBaseTest {

  public void testNewVersionable() throws Exception {

    Node node;
    try {
      node = root.addNode("node-v", "mix:versionable");
      fail("AddNode ConstraintViolationException should be thrown as type is not primary!");
    } catch (ConstraintViolationException e) {
    }

    node = root.addNode("node-v", "nt:unstructured");

    node.addMixin("mix:versionable");
    assertEquals(1, node.getMixinNodeTypes().length);
    assertEquals("mix:versionable", node.getMixinNodeTypes()[0].getName());
    assertEquals("nt:unstructured", node.getPrimaryNodeType().getName());

    assertNotNull(node.getProperty("jcr:uuid").toString());

    assertTrue(node.isNodeType("mix:versionable"));

    root.save();

    node = root.getNode("node-v");

    assertNotNull(node.getProperty("jcr:isCheckedOut"));
    assertTrue(node.getProperty("jcr:isCheckedOut").getBoolean());

    assertNotNull(node.getProperty("jcr:predecessors"));
    assertEquals(1, node.getProperty("jcr:predecessors").getValues().length);

    assertNotNull(node.getProperty("jcr:versionHistory"));
    String vhRef = node.getProperty("jcr:versionHistory").getString();

    assertNotNull(node.getProperty("jcr:baseVersion"));
    String bvRef = node.getProperty("jcr:baseVersion").getString();

    assertTrue(node.isNodeType("mix:versionable"));

    VersionHistory vh = node.getVersionHistory();
    System.out.println(" node " + node.getUUID() + " "
        + vh.getProperties().getSize());
    assertEquals(vh.getUUID(), vhRef);
    assertNotNull(vh.getVersionableUUID());
    assertNotNull(vh.getNode("jcr:rootVersion"));

    Version bv = node.getBaseVersion();
    assertEquals(bv.getUUID(), bvRef);
    assertNotNull(bv.getProperty("jcr:created"));

  }

  public void testNewVersionable1() throws Exception {

    Node node;

    node = root.addNode("node-v1", "nt:unstructured");
    
    root.save();
    
    // another  session
    SessionImpl session1 = (SessionImpl)repository.login(new CredentialsImpl("exo", "exo".toCharArray()));
    
    Node root1 = session1.getRootNode();
    node = root1.getNode("node-v1");
    node.addMixin("mix:versionable");

    assertNotNull(node.getProperty("jcr:versionHistory"));

    assertTrue(node.isNodeType("mix:versionable"));
    
    //NodeDumpVisitor v = new NodeDumpVisitor();
    //node.accept(v);
    //log.debug("X "+node.getProperty("jcr:baseVersion"));
    //log.debug("XYYY >>> "+v.getDump());
    //log.debug("XYYYYYYYYYYYYYYYY "+session1.getTransientNodesManager().dump());
    
    root1.save();
    
    node = root1.getNode("node-v1");
    
    assertNotNull(node.getVersionHistory());

  }

  public void testCheckinAfterNewVersionable() throws Exception {
    Node node = root.addNode("node1", "nt:unstructured");

    node.addMixin("mix:versionable");

    root.save();

    Session s1 = repository.login(this.credentials, "ws");

    Node node1 = s1.getRootNode().getNode("node1");
    assertTrue(node1.getProperty("jcr:isCheckedOut").getBoolean());
    assertTrue(node1.isCheckedOut());

    node1.checkin();

    assertFalse(node1.getProperty("jcr:isCheckedOut").getBoolean());
    assertFalse(node1.isCheckedOut());

    node1.checkout();

    assertTrue(node1.getProperty("jcr:isCheckedOut").getBoolean());
    assertTrue(node1.isCheckedOut());

  }
  

}