/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.chart.impl;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.Iterator;

import org.exoplatform.services.chart.Chart;
import org.exoplatform.services.chart.ChartService;
import org.exoplatform.services.chart.Data;
import org.exoplatform.services.chart.ExoCategoryItemLabelGenerator;
import org.exoplatform.services.download.DownloadService;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;
/**
 * Created by The eXo Platform SARL
 * Author : Hung Nguyen
 *          nqhungvn@users.sourceforge.net
 * Jul 18, 2005
 */
public class ChartServiceImpl implements ChartService {
  private DownloadService dservice_ ;
  final static private String DEFAULT_ID = "chart:/default" ;
  
  public ChartServiceImpl(DownloadService downloadservice) throws Exception {
    dservice_ = downloadservice ;
    ChartDownloadResource defaultChart = new ChartDownloadResource(createBlankChart()) ;
    dservice_.addDefaultDownloadResource(defaultChart) ;
  }
  
  public BufferedImage createChart(Chart chart) throws Exception {
    if(chart==null || chart.getChartData()==null || chart.getChartData().isEmpty()) return null;    
    if(Chart.PIE_CHART == chart.getChartType()) {
      return createPieChart(chart);
    }
    if(Chart.COLUMN_CHART == chart.getChartType()) {
      return createBarChart(chart);
    }
    return null;
  }
  
  public String createDownloadableChart(Chart chart) throws Exception {
    if (chart==null || chart.getChartData()==null ||chart.getChartData().isEmpty()) return DEFAULT_ID;
    BufferedImage image = createChart(chart) ;
    ChartDownloadResource cresource = new ChartDownloadResource(image) ;
    String downloadId = dservice_.addDownloadResource(cresource) ;        
    return downloadId ;
  }
  
  public String creatDownloadableChartLink(Chart chart) throws Exception {
    String chartlink = null ;
    if (chart==null || chart.getChartData()==null ||chart.getChartData().isEmpty()){
      chartlink = dservice_.getDownloadLink(DEFAULT_ID) ;
      return chartlink; 
    }
    BufferedImage image = createChart(chart) ;
    ChartDownloadResource cresource = new ChartDownloadResource(image) ;
    String resouceId = dservice_.addDownloadResource(cresource) ;
    chartlink = dservice_.getDownloadLink(resouceId) ;
    return chartlink ;
  }
  public BufferedImage  getDownloadableChart(String chartId)  throws Exception  {
    ChartDownloadResource dresource =
      (ChartDownloadResource)dservice_.getDownloadResource(chartId) ;    
    return dresource.getChartImage() ;
  }
  
  private BufferedImage createPieChart(Chart chart) {
    DefaultPieDataset dataset = new DefaultPieDataset();
    Iterator it =  chart.getChartData().iterator();
    JFreeChart backChart;    
    while(it.hasNext()) {
      Data data = (Data)it.next();
      dataset.setValue(data.getType(), data.getValue());
    }    
    //Create chart
    if(chart.getIs3DChart()){
      backChart = ChartFactory.createPieChart3D(chart.getChartTitle(),dataset,true, true, false);      
    }else{
      backChart = ChartFactory.createPieChart(chart.getChartTitle(),dataset,true, true, false);
    }
    backChart.setBackgroundPaint(chart.getBackGroundColor());
    PiePlot plot = (PiePlot)backChart.getPlot();    
    //Start point
    plot.setStartAngle(chart.getStartAngle());
    //Display color
    it = chart.getChartData().iterator();
    while(it.hasNext()){
      Data data = (Data)it.next();
      if(data.getDisplayColor()!= null) plot.setSectionPaint(data.getPainPosition(), data.getDisplayColor());
    }
    //Direct rotation
    plot.setDirection(chart.getDirection());
    //Foreground
    if (chart.getIsForeGround()) plot.setForegroundAlpha(0.5f);
    return backChart.createBufferedImage(chart.getChartWidth(), chart.getChartHeight());
  }
  
  private BufferedImage createBarChart(Chart chart) {
    DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    PlotOrientation orientation;
    JFreeChart backChart;
    Iterator it =  chart.getChartData().iterator();
    while(it.hasNext()) {
      Data data = (Data)it.next();
      dataset.setValue(data.getValue(), data.getSeries(), data.getType());      
    }    
    //Set chart Orientation
    if(chart.getIsHorizontal()) { orientation = PlotOrientation.HORIZONTAL; }
    else{ orientation = PlotOrientation.VERTICAL; }
    //Create chart
    if(chart.getIs3DChart()) {
      backChart = ChartFactory.createBarChart3D(chart.getChartTitle(), chart.getCategoryLabel(), chart.getValueLabel(), dataset, orientation, true, true, false); 
    }else {
      backChart = ChartFactory.createBarChart(chart.getChartTitle(), chart.getCategoryLabel(), chart.getValueLabel(), dataset, orientation, true, true, false);
    }
    CategoryPlot plot = backChart.getCategoryPlot();
    CategoryItemRenderer renderer = plot.getRenderer();
    if (chart.getIsDisplayItemLabel()) {
      renderer.setItemLabelGenerator(new ExoCategoryItemLabelGenerator());
      renderer.setItemLabelsVisible(true);
    }
    // Set display color for bar
    it = chart.getChartData().iterator();
    while(it.hasNext()){
      Data data = (Data)it.next();
      if(data.getDisplayColor()!= null) renderer.setSeriesPaint(data.getPainPosition(), new GradientPaint( 0.0f, 0.0f, data.getDisplayColor(), 0.0f, 0.0f, data.getDisplayColor()));
    }
    return backChart.createBufferedImage(chart.getChartWidth(), chart.getChartHeight());    
  }
  
  private BufferedImage createBlankChart(){
    String errorString = "Chart's data does not correct, please back to list cache and click [ view ] button again!" ;
    BufferedImage image = new BufferedImage(600,400 , BufferedImage.TYPE_INT_RGB);
    Graphics2D offscreen = (Graphics2D)image.getGraphics() ;
    offscreen.setColor(Color.lightGray);
    offscreen.fillRect(0, 0, 600, 400);
    offscreen.setColor(Color.black);
    offscreen.drawString(errorString, 70, 200);
    return image ;
  }  
}
