/***************************************************************************
 * Copyright 2001-2003 The VietSpider Studio        All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.List;

import javax.naming.NamingEnumeration;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.Control;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.PagedResultsControl;
import javax.naming.ldap.PagedResultsResponseControl;
import javax.naming.ldap.SortControl;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserHandler;

/**
 * Created by VietSpider Studio
 * Author : Nhu Dinh Thuan
 *          nhudinhthuan@yahoo.com
 * Dec 7, 2005
 */
public class LDAPUserPageList extends PageList {  
 
  private String searchBase_;
  private String filter_;
  private LDAPService ldapService_;
  private LDAPAttributeMapping ldapAttrMapping_;
  private UserHandler handler_;
  private boolean controlCritical_ = false;
  
  public LDAPUserPageList(UserHandler handler, LDAPAttributeMapping ldapAttrMapping,
      LDAPService ldapService,String searchBase,String filter,int pageSize, boolean controlCritical) throws Exception {   
    super( pageSize);
    this.ldapAttrMapping_ = ldapAttrMapping;
    this.ldapService_= ldapService;
    this.searchBase_ = searchBase;
    this.filter_ = filter;
    this.handler_ = handler;   
    this.controlCritical_ = controlCritical;
    int size = this.getResultSize();     
    this.setAvailablePage( size);   
  }
  
  protected void populateCurrentPage(int page) throws Exception  {
    List<User> users = new ArrayList<User>(); 
    PagedResultsControl prc = new PagedResultsControl( this.getPageSize(), Control.CRITICAL) ; 
    String keys[] = { ldapAttrMapping_.userUsernameAttr };
    SortControl sctl = new SortControl(keys, controlCritical_);
    
    LdapContext ctx = ldapService_.getLdapContext();
    ctx.setRequestControls(new Control[] { sctl, prc });
    SearchControls constraints = new SearchControls();
    constraints.setSearchScope( SearchControls.SUBTREE_SCOPE);
    
    byte[] cookie = null;
    int counter = 0;
    
    do {
      counter++;
      NamingEnumeration<SearchResult> results = ctx.search( searchBase_, filter_, constraints);    
      if( results == null) break;     
      while (results.hasMore()){
        SearchResult result = results.next();
        if( counter == page)         
          users.add( ldapAttrMapping_.attributesToUser( handler_, result.getAttributes()));        
      }      
      Control[] responseControls = ctx.getResponseControls();      
      for (int z = 0; z < responseControls.length; z++) {
        if (responseControls[z] instanceof PagedResultsResponseControl)                 
          cookie = ((PagedResultsResponseControl)responseControls[z]).getCookie();         
      }
      ctx.setRequestControls(new Control[]{
          new PagedResultsControl( this.getPageSize(), cookie, Control.CRITICAL)});     
    } while ( cookie != null);   
    this.currentListPage_ = users;
  }
  
  private int getResultSize() throws Exception {    
    PagedResultsControl prc = new PagedResultsControl( this.getPageSize(), Control.CRITICAL) ; 
    String keys[] = { ldapAttrMapping_.userUsernameAttr };
    SortControl sctl = new SortControl(keys, controlCritical_);
    
    LdapContext ctx = ldapService_.getLdapContext();
    ctx.setRequestControls(new Control[] { sctl, prc });
    SearchControls constraints = new SearchControls();
    String returnedAtts[]={ ldapAttrMapping_.userUsernameAttr};
    constraints.setReturningAttributes(returnedAtts);
    constraints.setSearchScope( SearchControls.SUBTREE_SCOPE);
    
    byte[] cookie = null;
    int counter = -1;
    
    do {      
      NamingEnumeration<SearchResult> results = ctx.search( searchBase_, filter_, constraints);    
      if( results == null) break;     
      while (results.hasMore()){
        counter++;
        results.next();
      }
      
      Control[] responseControls = ctx.getResponseControls();      
      for (int z = 0; z < responseControls.length; z++) {
        if (responseControls[z] instanceof PagedResultsResponseControl)                 
          cookie = ((PagedResultsResponseControl)responseControls[z]).getCookie();         
      }
      ctx.setRequestControls(new Control[]{
          new PagedResultsControl( this.getPageSize(), cookie, Control.NONCRITICAL)});
    } while ( cookie != null);   
    return  counter;      
  }
  
  public List getAll() throws Exception  { 
    return null;
  }  

  
}
