/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.templates.groovy;

import groovy.text.Template;

import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.test.BasicTestCase;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 16 juin 2004
 */
public class TestGroovyService extends BasicTestCase {
  private GroovyTemplateService service_ ;
  
  public TestGroovyService(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    service_ = 
      (GroovyTemplateService) PortalContainer.getInstance().
       getComponentInstanceOfType(GroovyTemplateService.class) ;
  }

  public void testGroovyService() throws Exception {
    String basedir = System.getProperty("basedir") ;    
    GroovyTemplateFileLoader floader = new GroovyTemplateFileLoader(basedir + "/src/resource") ;
    floader.setName("TemplateFileLoader") ;
    assertTrue("expect TemplateFileLoader can load",
                floader.canLoad("file","FileLoaderTemplate.gtmpl") && 
                floader.load("FileLoaderTemplate.gtmpl")!=null) ;    
    assertEquals("expect TemplateFileLoader's priority is NOMAL",
                  floader.getPriority(),GroovyTemplateLoader.NORMAL_PRIORITY) ;
    floader.setPriority(GroovyTemplateLoader.HIGH_PRIORITY) ;
    assertEquals("now, expect TemplateFileLoader's priority is updated",
                  floader.getPriority(),GroovyTemplateLoader.HIGH_PRIORITY) ;    
    service_.addTemplateLoader(floader) ;
    
    GroovyTemplateClasspathLoader cploader = new GroovyTemplateClasspathLoader() ;
    cploader.setName("ClasspathLoaderTemplate") ;
    String cpresource ="ClasspathLoaderTemplate.gtmpl" ;
    assertTrue("expect TemplateClasspathLoader can load",
                cploader.canLoad("classpath",cpresource) && 
                cploader.load(cpresource)!=null) ;    
    service_.addTemplateLoader(cploader) ;
    
//    String templateTextLoader = "this is Text Template Loader $hello:special" ;
//    GroovyTemplateTextLoader txtloader = new GroovyTemplateTextLoader("text",templateTextLoader) ;    
//    assertTrue("expect TemplateTextLoader cand load",
//                txtloader.canLoad("text",templateTextLoader) && 
//                txtloader.load(templateTextLoader)!=null) ;
//    service_.addTemplateLoader(txtloader) ;
    
    Map binding = new HashMap() ;
    binding.put("helloClasspathLoader", "hello classpath loader") ;
    binding.put("helloFileLoader", "hello file loader") ;
    binding.put("$hello:special", "hello special") ;    
    
    StringWriter w = new StringWriter() ;    
    service_.merge(w, "file", "FileLoaderTemplate.gtmpl", binding) ;
    String fresult = w.getBuffer().toString() ;    
    assertTrue(fresult.indexOf("This  is a  file loader template") >= 0) ;
    
    String cpresult = service_.merge("classpath",cpresource,binding) ;
    assertTrue(cpresult.indexOf("This is a classpath loader template") >= 0) ;
    
//    service_.merge(w,"text",templateTextLoader,binding) ;
//    String txtresult = w.getBuffer().toString() ;
//    System.out.println(txtresult) ;
  } 
}