/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.ResourceBundle;

import javax.faces.component.UIComponent;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.model.Page;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPageForm.java,v 1.8 2004/09/29 17:44:51 benjmestrallet Exp $
 */
public class UICopyPageForm extends UISimpleForm {
  final static String PAGE_OWNER="pageOwner", PAGE_NAME="pageName", DELETE_PAGE="deletePage",
                      PAGE_TITLE="pageTitle";
  
  private Page copyPage_;
  private PortalConfigService service_ ;
  private boolean adminOwner_ ;
  private UIPageList uiList_ ;
  private String curPageId_ ;
  
  public UICopyPageForm() throws Exception {
    super("copyPageForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer") ;
    setId("UICopyPageForm");
    adminOwner_ = false; 
    add(new UIStringInput(PAGE_OWNER, "")) ;
    add(new UIStringInput(PAGE_NAME, ""));
    add(new UIStringInput(PAGE_TITLE, ""));
    add(new UICheckBox(DELETE_PAGE, "true")) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    
  } 
  
  public void setCopyPage(UIComponent returnComponent, Page newPage, String curPageId) throws Exception{
    uiList_ = (UIPageList)returnComponent ;
    copyPage_ = newPage ;
    curPageId_ = curPageId ;
    UIPageBrowser uiPageBrowser = (UIPageBrowser) uiList_.getParent();
    UIPortal uiPortal = (UIPortal) uiPageBrowser.getAncestorOfType(UIPortal.class) ;
    service_ = uiPageBrowser.getPortalConfigService();
    //Page model = service.getPage(pageId);
    getUIStringInput(PAGE_OWNER).setValue(copyPage_.getOwner()) ;
    getUIStringInput(PAGE_NAME).setValue(copyPage_.getName());
    getUIStringInput(PAGE_TITLE).setValue(copyPage_.getTitle());
    if("admin".equals(uiPortal.getOwner()))
      adminOwner_ = true ;
    else
      getUIStringInput(PAGE_OWNER).setReadonly(true) ;        
  } 
  
  public boolean getOwner() { return adminOwner_ ; }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICopyPageForm uiForm = (UICopyPageForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class) ;
      uiPortal.setBodyComponent(uiForm.uiList_.getParent());
    }
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICopyPageForm uiForm = (UICopyPageForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class) ;
      String id = uiForm.copyPage_.getOwner()+":"+uiForm.copyPage_.getName() ;
      String newId = uiForm.getUIStringInput(PAGE_OWNER).getValue() + ":" + uiForm.getUIStringInput(PAGE_NAME).getValue();
      if(id.equals(newId)) {
        Information iprovider = findInformationProvider(uiForm);
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UICopyPage.msg.duplicate-id"))) ;
        return ;
      }
      if(uiForm.adminOwner_){
        if(uiForm.getUICheckBox(DELETE_PAGE).getSelect()){
          UIPageBrowser uiPageBrowser = (UIPageBrowser) uiForm.uiList_.getParent();
          PortalConfigService service = uiPageBrowser.getPortalConfigService();
          service.removePage(uiForm.curPageId_) ; 
          PortalComponentCache  cache = 
          (PortalComponentCache) SessionContainer.getComponent(PortalComponentCache.class) ; 
          cache.removeUIPageFromCache(uiForm.curPageId_) ;
        }
      }
      uiForm.copyPage_.setOwner(uiForm.getUIStringInput(PAGE_OWNER).getValue());
      uiForm.copyPage_.setName(uiForm.getUIStringInput(PAGE_NAME).getValue());
      uiForm.copyPage_.setTitle(uiForm.getUIStringInput(PAGE_TITLE).getValue());
      uiForm.service_.savePage(uiForm.copyPage_);
      uiForm.uiList_.update() ;
      uiPortal.setBodyComponent(uiForm.uiList_.getParent());
    }
  }  
  
}