/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.*;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.validator.ValidUserValidator;
import org.exoplatform.portal.Information;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.portal.community.CommunityConfigService;
import org.exoplatform.services.portal.community.CommunityNavigation;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIGroupForm.java,v 1.11 2004/10/21 15:25:17 tuan08 Exp $
 */
public class UICommunityNavForm extends UISimpleForm {
  static String MEMBERSHIP = "membership" , NAVIGATION = "navigation", 
                DESCRIPTION = "description", PRIORITY = "priority" ;
  static public List PRIORITIES ;
  static {
    PRIORITIES = new ArrayList() ;
    for(int i = 1; i <= 10; i++) {
      String num = Integer.toString(i) ;
      PRIORITIES.add(new SelectItem(num, num)) ;
    }
  }
  
  private CommunityNavigation communityNav_ ;
  
  public UICommunityNavForm(OrganizationService orgService) throws Exception {
    super("communityNavForm", "post", null) ;
    setRendererType("SimpleFormVelocityRenderer") ;
    Collection memberships = orgService.getMembershipTypeHandler().findMembershipTypes() ;
    Iterator i = memberships.iterator() ;
    List mtypes = new ArrayList(10) ;
    while(i.hasNext()) {
      MembershipType mt = (MembershipType) i.next();
      String name = mt.getName() ;
      mtypes.add(new SelectItem(name, name)) ;
    }
    add(new UISelectBox(MEMBERSHIP, "", mtypes)) ;    
    add(new UIStringInput(NAVIGATION, "").addValidator(ValidUserValidator.class)) ;
    add(new UISelectBox(PRIORITY, "1", PRIORITIES)) ;  
    add(new UITextArea(DESCRIPTION, "")) ;
    
    addActionListener(SaveUpdateListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }
 
  public void setCommunityNavigation(CommunityNavigation cn) {
    communityNav_ = cn;
    if(cn == null)  {
      reset() ;
    } else {
      getUISelectBox(MEMBERSHIP).setValue(cn.getMembership()) ;
      getUISelectBox(PRIORITY).setValue(Integer.toString(cn.getPriority())) ;
      getUIStringInput(NAVIGATION).setValue(cn.getNavigation()) ;
      getUIStringInput(DESCRIPTION).setValue(cn.getDescription()) ;
    }
  }
  
  static public class SaveUpdateListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UICommunityNavForm uiForm = (UICommunityNavForm) event.getComponent() ;
      CommunityConfigService service = 
        (CommunityConfigService)PortalContainer.getComponent(CommunityConfigService.class) ;
      CommunityNavigation cn = uiForm.communityNav_ ;
      if(cn == null) {
        UIGroupExplorer uiExplorer = 
          (UIGroupExplorer) uiForm.getAncestorOfType(UIGroupExplorer.class) ;
        Group currentGroup = uiExplorer.getCurrentGroup() ;
        if(currentGroup == null) {
          Information iprovider = findInformationProvider(uiForm);
          ResourceBundle res = getApplicationResourceBundle() ;
          iprovider.addMessage(new ExoFacesMessage(res.getString("UICommunityNavForm.msg.no-group"))) ;
          uiForm.setRenderedSibling(UIGroupCommunityInfo.class) ;
          return ;
        } 
        cn = service.createCommunityNavigation() ;
        cn.setGroupId(currentGroup.getId()) ;
      }
      cn.setMembership(uiForm.getUISelectBox(MEMBERSHIP).getValue()) ;
      cn.setNavigation(uiForm.getUIStringInput(NAVIGATION).getValue()) ;
      cn.setPriority(Integer.parseInt(uiForm.getUISelectBox(PRIORITY).getValue())) ;
      cn.setDescription(uiForm.getUIStringInput(DESCRIPTION).getValue()) ;
      service.addCommunityNavigation(cn) ;
      UIGroupCommunityInfo uiInfo = 
        (UIGroupCommunityInfo) uiForm.getSibling(UIGroupCommunityInfo.class);
      uiInfo.setCommunityNavigation(cn) ;
      uiForm.setRenderedSibling(UIGroupCommunityInfo.class) ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
      UICommunityNavForm uiForm = (UICommunityNavForm) event.getComponent() ;
      uiForm.setRenderedSibling(UIGroupCommunityInfo.class) ;
    }
  }
}