/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.portletcontainer.impl.servlet;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.exoplatform.container.RootContainer;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portletcontainer.PortletApplicationRegister;
import org.exoplatform.services.portletcontainer.pci.model.PortletApp;
import org.exoplatform.services.portletcontainer.pci.model.XMLParser;
import org.exoplatform.services.xml.resolving.SimpleResolvingService;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
/**
 * Created by the Exo Development team.
 * Author : Mestrallet Benjamin
 * benjmestrallet@users.sourceforge.net
 * Date: 10 nov. 2003
 * Time: 12:58:52
 */
public class PortletApplicationListener implements ServletContextListener {
  public void contextInitialized(ServletContextEvent servletContextEvent) {
    RootContainer manager = RootContainer.getInstance();
    LogService lservice = (LogService)manager.getComponentInstanceOfType(LogService.class) ;
    Log log = lservice.getLog("org.exoplatform.services.portletcontainer");
    ServletContext servletContext = servletContextEvent.getServletContext();
    log.info("DEPLOY PORTLET APPLICATION: " + servletContext.getServletContextName());
    log.debug("Real path : "+ servletContext.getRealPath(""));   
    InputStream is = null;
    String oldParser = System.getProperty("javax.xml.parsers.DocumentBuilderFactory") ;
    try {
      is = servletContext.getResourceAsStream("/WEB-INF/portlet.xml");
      if (is == null) {
        log.info("PORTLET CONFIGURATION IS NOT FOUND, IGNORE THE PACKAGE");
        return;
      }
      PortletApp portletApp = XMLParser.parse(is);
      
      is = servletContext.getResourceAsStream("/WEB-INF/web.xml");
      Collection<String> roles = new ArrayList<String>();
      
      XPath xpath = XPathFactory.newInstance().newXPath();
      XPathExpression roleNameExp = xpath.compile("/web-app/security-role/role-name") ;
      DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
      SimpleResolvingService serviceXML = 
          (SimpleResolvingService) manager.getComponentInstanceOfType(SimpleResolvingService.class);
      builder.setEntityResolver(serviceXML.getEntityResolver());
      Document document = builder.parse(is);
      NodeList nodes = (NodeList) roleNameExp.evaluate(document , XPathConstants.NODESET);
      for (int i = 0; i <  nodes.getLength(); i++) {
        Node element = nodes.item(i);
        roles.add(element.getFirstChild().getNodeValue());
        System.out.println("===>  NODE VALUE : "  + element.getFirstChild().getNodeValue()) ;
      }
            
      PortletApplicationRegister service = 
        (PortletApplicationRegister) manager.getComponentInstanceOfType(PortletApplicationRegister.class);
      service.registerPortletApplication(servletContext, portletApp, roles); 
    } catch (Exception e) {
      log.error("Cannot deploy " + servletContext.getServletContextName(), e);
    } finally {
      if(oldParser != null) {
        System.setProperty("javax.xml.parsers.DocumentBuilderFactory", oldParser) ;
      }
    }
  }

  public void contextDestroyed(ServletContextEvent servletContextEvent) {
    ServletContext servletContext = servletContextEvent.getServletContext();
    RootContainer manager = RootContainer.getInstance();
    LogService lservice = (LogService)manager.getComponentInstanceOfType(LogService.class) ;
    Log log = lservice.getLog("org.exoplatform.services.portletcontainer");
    log.info("UNDEPLOY PORTLET APPLICATION: " + servletContext.getServletContextName());    
    try {
      PortletApplicationRegister service = 
        (PortletApplicationRegister) RootContainer.getInstance().getComponentInstanceOfType(PortletApplicationRegister.class);
      service.removePortletApplication(servletContext);
    } catch (Exception e) {
      log.error(e);
    }
  }
}